function [DRT,df,dl,ddf,l] = Ambiguity(ref)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    08112016
%%  Free to use for non-commercial, academic or Standards Body use
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Computes the Auto-correlation and Cross-Ambiguity Function for the
%% SRRC filtered over-sampled input sequence based on an assumed Baud = 19.2K
%% and an OSR = 6.  Normalizes to peak Auto-correlation response and then 
%% decimates by 2 in both dimensions to facilitate Pk/SL search.
%%
%% Outputs:
%%      DRT - the decomated dB scale Cross-Ambiguity Function
%%      df - the undecimated frequency points
%%      dl - the decimated lag value indices
%%      ddf - the decimated frequency points
%%      l - the undecimated lag values in samples at fs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
OSR = 6;
BR = 19200;
fs = OSR*BR;
NT = 6;
pi2 = 2*pi;
% 
d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,1.25-1);
H = design(d);
hsrrc = H.numerator;
hsrrc = hsrrc/sum(hsrrc);
ndelay = (length(hsrrc)-1)/2;
save ShapingFIR hsrrc ndelay;

load ShapingFIR;

iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));
sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
sref = sref(ndelay+1:end);
% sref = kron(ref,ones(1,OSR));
[r,l] = xcorr(sref,sref);
pk0 = max(abs(r));
% figure(3)
% plot((l/OSR),dbp(abs(r)/pk0))
% grid
% xlabel('Time Offset in Symbols')
% ylabel('Auto-Correlation Response (dB)')
% title('Currently porposed Barker-based VDE Training Sequence')

df = -5000:10:5000;
lf = length(df);
for m=1:lf
    dref = sref.*exp(1i*pi2*df(m)/fs*(0:length(sref)-1));
    [dr,l]=xcorr(dref,sref);
    DR(m,:) = abs(dr);
%     DR(m) = abs(dref.*conj(sref));
%     plot(l/OSR,abs(dr)/pk0);
%     hold on;
%     pause(0.1);
end

PK = max(max(DR));
dfmax = 5000;
tsmax = floor(max(l)/OSR*0.8);
ddf = find(abs(df)<=dfmax);
ddf = ddf(1:2:end);
dl = find(abs(l)/OSR<=tsmax);
dl = dl(1:2:end);
DRT = dbp(DR(ddf,dl)/PK);
DRT(DRT<-32) = -32;

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%