%
% FSK modulation testing - A. Macikunas, Waves in Space Corp.
% May 2017
%
runcode = 0;

if( runcode == 1),
    freqsep = 8;  % Frequency separation (Hz)
    nsamp = 8;    % Number of samples per symbol
    Fs = 32;      % Sample rate (Hz)
    % Generate random M-ary symbols.
    M = 4;        % Modulation order
    freqsep = 700;  % Frequency separation (Hz)
    nsamp = 8;    % Number of samples per symbol
    Fs = 9600;      % Sample rate (Hz)
    % Generate random M-ary symbols.
    
    x = randi([0 M-1],1000,1);
    % Apply FSK modulation.
    
    phase_cont = 'cont';
    symbol_order = 'grey';
    
    y = fskmod( x,M,freqsep,nsamp,Fs, phase_cont );
    % Create a spectrum analyzer System object? and use its step method to display a plot of the signal spectrum.
    
    figure(200); plot(real(y(1:200)),'k')
    figure(201); plot(180/pi*unwrap(angle(y(1:200))),'.')
    
    h = dsp.SpectrumAnalyzer('SampleRate',Fs);
    
    step(h,y)
    
else
    
    % Set the simulation parameters - from original example
    if(1==0)
        M = 2;         % Modulation order
        k = log2(M);   % Bits per symbol
        EbNo = 5;      % Eb/No (dB)
        Fs = 16;       % Sample rate (Hz)
        nsamp = 8;     % Number of samples per symbol = oversampling rate, OS
        freqsep = 10;  % Frequency separation (Hz)
    else
        %  Arunas parameters - modified for training sequence
        M = 4;         % Modulation order
        k = log2(M);   % Bits per symbol
        EbNo = 5;      % Eb/No (dB)
%         Fs = 9600;     % Sample rate (Hz)
        SymbR = 19200;  % symbol rate (Hz)
        OSR     = 20;   % oversampling rate
        Fs = SymbR * OSR;    % Sample rate (Hz)

        nsamp = Fs/SymbR;     % Number of samples per symbol
%         freqsep = SymbR/8;  % Frequency separation (Hz)
        freqsep = 4* 1296;  % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X

%         freqsep = 2400;  % Frequency separation (Hz)
    end
    numbersymb  = 27;
    numbit      = numbersymb * M;
    
    % Generate random data symbols.
    if(1==1)
%         data = randi([0 M-1],numbit,1)';
%          data = [0 0 0 0 0 0 1 2 3 0 0 2 2 1 3 0 3 1 1 2 0 3 3 2 1];
%         data = [0 1 2 3 2 0 1 3 ];
%         data = [0 1 2 3 2 0 1 3 ];
        data = randi([0 M-1],numbersymb,1)';
        data = 2*[ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ]; %space Norway
        data = [ 1 1 1 0 0 0 0 1 1 0 1 1 0 1 1 0 0 1 0 0 1 1 1 1 0 1 0 ]; % Arunas 1
        data = [ 2 1 1 3 0 0 3 1 1 3 1 1 3 1 1 3 0 1 3 0 1 2 1 1 3 1 0 ]; % Arunas 2
        data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 1 4 ] - 1; % Arunas 3 not bad, ISL -4.1 dB
        
        data = randi([0 M-1],numbersymb,1)';
        data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 ]; % Arunas 4 1st 24-symbols from DMR voice BS bound
        data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 0 2 0 2 0]; % Arunas 5 1st 24-symbols from DMR voice BS bound + AM

    else
        data = randi([0 M-1],5000,1);
    end
    % Apply FSK modulation.
    
    txSig = fskmod(data,M,freqsep,nsamp,Fs);
    % Pass the signal through an AWGN channel
    
    rxSig  = awgn(txSig,EbNo+10*log10(k)-10*log10(nsamp),...
        'measured',[],'dB');
    % Demodulate the received signal.
    
    dataOut = fskdemod(rxSig,M,freqsep,nsamp,Fs);
    % Calculate the bit error rate.
    
    [num,BER] = biterr(data,dataOut);
    % Determine the theoretical BER and compare it to the estimated BER. Your BER value might vary because the example uses random numbers.
    
    tx_padded = [ 0*ones(1,2048-length(txSig)) txSig];
    figure(50); plot(real(txSig),'.'); hold on; plot(real(rxSig),'.'); hold off; grid
%     figure(51); plot( 20*log10(abs(fftshift(fft(txSig)))),'.');
        figure(51); plot( 20*log10(abs(fftshift(fft(tx_padded)))),'.');

    BER_theory = berawgn(EbNo,'fsk',M,'noncoherent');
    
    [BER BER_theory]
    save txSigref txSig M k Fs nsamp freqsep
    
end