%% Working Group 5 - Training Sequence Analysis
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    07112016
%%  Additions and edits A. Macikunas, 30052017, 01062017, 04062017
%%  Free to use for non-commercial, academic or Standards Body Purposes
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Implements a poor-mans ROC curve estimation for a specific Tarining sequence
%% in a QPSK Modulated message with a BPOSK or other form of training sequence
%% Additions to analysis, new waveforms, updated and new plots, SNR loop
%%   diagnostic data collection for examination of detector statistics AGM May 2017
%% Outputs:
%%     Plots ROC Curves for study, changed to fixed threshold ROC, remove CFAR estimation/threshold
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
clear all;
r2      = sqrt(2);
pi2     = 2*pi;

training_type = 1;
% training_type = menu(' Training sequence to run', 'Barker sequence', 'Zadoff Chu 27 sequence', 'Space Norway 27-bit sequence');
training_type = menu('Select Reference', ...
    'Barker (Jan coder)','13-bit Correlator Barker (Jan coder)','Space Norway 27-bit propsed', 'Zadoff Chu 27-symb', ...
    'Zadoff Chu 32-symb', 'Chu 27-symb', 'Chu 32-symb','Polyphase Barker 27 (TD)','Frank 25', ...
    'Unattributed binary 27 (TD)', 'Unfiltered Zadoff-Chu 27', 'AM 69 phase sequence', ...
    '4 FSK Random', '4 FSK like Frank', '4 FSK future', 'Future - various to try');

% load VDE_TrngBarker;            %% sref @ OSR = 8;
if(training_type==1)
    %% Define the proposed Barker-based Training Sequence
    if(1==0)
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];       % Tim's original
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
        TRNG = [1 barker ~barker];
        symbols = [TRNG; TRNG];
        TrngSym = [2 1]*symbols;
        k3 = find(TrngSym==3);
        TrngSym(k3) = 2;
        lTrng = length(TrngSym);
        TRNG = exp(1i*(TrngSym*pi/2+pi/4));     % Tim's original
    else
        %  check if this is still correct, as used in other code - AGM 27 May 2017
        %  ************************************************
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
        TRNG = [1 barker ~barker];         %% As currently proposed
        
        trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
        trng_mapped(2:2:2*length(TRNG)) = TRNG;
        
        trng_mapped;
        
        TRNG = qpskMapperJan(trng_mapped);
    end
    
    seq_type = 'Barker QPSK Jan';
elseif(training_type==2)
    fprintf('FUTURE - not supported yet \n');
    
elseif(training_type==3)
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;     % use same symbol mapping as Barker AGM May 2017
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    TRNG = qpskMapperJan(trng_mapped);
    seq_type = 'Space Norway 27 symb';
    
elseif(training_type==4)
    %    load ZadofChu_AmbxxUnFilt; TRNG = bb;  %% "best" 27 symbol Choice so far: Unfiltered
    load ZadofChu_Amb; TRNG = bb;            %
    
    seq_type = 'ZadoffChu 27 symb';
    
elseif(training_type==5)
    
    load ZadofChu32_Ambxy; TRNG = bb;
    seq_type = 'ZadoffChu32-Ambxy';
    
elseif(training_type==6)
    load CHU_27; TRNG = a;  % am - does this exist - need to create it
    seq_type = 'Chu-27';
    
elseif(training_type==7)
    
    load CHU_32; TRNG = a;  % am - does this exist - need to create it
    seq_type = 'Chu-32';
    
elseif(training_type==8)
    
    load polyphase_barker27; TRNG = bb;
    seq_type = 'Polyphase Barker Pi/4-QPSK';
    
elseif(training_type==9)
    
    Frank25 = [0 0 0 0 0 0 1 2 3 4 0 2 4 1 3 0 3 1 4 2 0 4 3 2 1];
    TRNG = exp(1i*pi2*Frank25/5);
    seq_type = 'Frank25';
    
elseif(training_type==10)
    
    [TRNG, ~] = pi4qpskMapper([0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1], 0);
    
    seq_type = 'Unlabelled binary (TD)';
    
elseif(training_type==11)
    
    load ZadofChu_AmbxxUnFilt; TRNG = bb;    %% "best" Choice so far: Unfiltered
    
    seq_type = 'ZadoffChu-27 Unfiltered';
    
elseif(training_type==12)
    if(1==0)
        phasefn = pi/180* [143.999883964013,157.998625492865,80.6601383580463,125.505473402409,-47.7831995128548,-38.6672199679412,-155.433191098205,-172.904511808408,-73.0862794751650,-59.9186651412132,161.070957430118,142.842665670704,172.339404074901,47.3647678495325,-4.22531712412601,-57.6785977737499,-177.106307110185,-115.317826299311,-76.4249872526181,6.95616231697665,168.515635982310,130.735749482123,88.4216148241380,-9.88227000210407,-172.728430511086,-112.500295778172,-41.3682672125228,59.3779378548736,179.987676498986,52.4693247207682,-59.9010382147719,-131.336241637577,-156.935563951675,18.7964507151681,178.958252560295,59.9624047016509,-69.8217124160105,-145.171386886692,-136.651141316970,-46.1722826285207,84.3698167853535,171.780529683638,136.801341571765,-18.4056301772432,-179.991044342716,-20.7104495328089,140.640254603021,174.783046724681,61.7102983929129,-97.7770789359145,-166.119694584109,-84.7679220783924,72.3985742161437,160.137571571137,93.8285141569285,-64.9949252277021,-159.641713782967,-90.9131014605413,75.1464427376828,164.658256243350,75.9817911374775,-101.901535392152,-172.775415552544,-47.2733157442497,143.504431484207,179.296300532375,2.04059699920865,-16.6553996772684,-177.411658954999];
    else
        load AMphase
    end
    TRNG = exp(1i*pi2*phasefn);
    seq_type = 'AM 69 phase sequence';
    
elseif(training_type==13)
    
    load txSigref txSig M
    TRNG = txSig.';
    TRNG = txSig;
    shapingf = 0.5;
    
    seq_type = '4FSK random';
    
elseif(training_type==14)
    seq_type = '4FSK like Frank';
    
elseif(training_type==15)
    seq_type = '4FSK future';
    
elseif(training_type==16)
    
    % Files I don't have from Tim
    %     load Max27P2SL_1; ref = pn0;
    %     load Max32P2SL_TAG3; ref = pn0;
    %     load BestZadoff_ChuRev3; ref = bb;
    %     load BestZadoff_ChuRev3; ref = bb;     %% bb Second-BEST - small search range;
    %     load CHU_33;ref = a;                %% CAZAC Sequence
    %     load tempCHUxy;ref = bb;
    %     load tempChu1; ref = bb;              %% Chu BEST;
    % load tempCHUxx1; ref = bb;            %% very Good Intermediate: Unfiltered
end
lTrng = length(TRNG);

%% ---------------------------------------------------
body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));
body(2:2:end)   = body(2:2:end)*exp(1i*pi/4);

OSR                 = 8;
NT                  = 6;         %% Set-up the SRRC Shaping filter
alpha               = 0.3;    %% BW Excess factor
d                   = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,alpha);
H                   = design(d);
hsrrc               = H.numerator;
hsrrc               = hsrrc/sum(hsrrc);
ndelay              = (length(hsrrc)-1)/2;

inBB                = [TRNG body];     %% Test Message at BB
xin                 = kron(inBB,ones(1,OSR));
sig                 = filter(hsrrc,1,[xin zeros(1,ndelay)]);

sig                 = sig(ndelay+1:end);    %% Shaped, modulated Test Waveform
lsig                = length(sig);
ltrain              = lTrng*OSR;
ref                 = sig(1:ltrain);        %% Extract the ideal Training F for later detection processing

SNR                 = -1:1:5;               %% Set an SNR (Es/No) range for ROC Curve
Nsnr                = length(SNR);
Ntrials             = 100000;           %% Need to calcualte this for Statistical Significance
Ntrials         = 100;           %% Need to calcualte this for Statistical Significance

%% Typically, 90% Confidence intervals
k                   = 1;
Kt                  = 2.9417;                %% Detection threshold: based on Linear Envelope Detector (see Whalen's Text
%% Or use Robertson's curves for Deg of Freedom
KD                  = 0;
det                 = zeros(1,10);
far                 = det;
snr                 = 10^(SNR(2)/20);
snr = 10^(-3/20);
snr = 10^(0/20);

Ktmat               = 2:(1/3):7.25;

%% Create reference level and peak location - noiseless
x                   = [zeros(1,lsig) sig zeros(1,lsig)];     %% create message WF without noise for pk and std dev estimate
[r,l]               = xcorr(ref,x);       %% Cross Correlation Detection of Rx signal with itself
rstdRef             = std(r);
[rmaxRef, mxidxRef] = max(abs(r));
clear x r l

for Kt = 2:(1/3):7.25       %% Vary the detection threshold to sweep out PD v. PFA curve
    det(k) = 0;
    far(k) = 0;
    for j=1:Ntrials
        %         noise = cawgn(3*lsig)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No
        noise       = wgn(1,3*lsig,0)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No Modifed AGM used Matlab White Gaussian Noise, normalized power, lenght = arg
        
        
        x           = noise + [zeros(1,lsig) sig zeros(1,lsig)];     %% place message WF in AWGN
        
        [r,l]       = xcorr(x,ref);       %% Cross Correlation Detection of Trng Seq
        
        [mx, mxidx] = max(abs(r));
        rmax(k,j)   = mx;
        rmaxidx(k,j)= mxidx - 40703;            % empirical center index
        %         rmaxidx(k,j) = mxidx - mxidxRef;
        
        y = abs(r(l>-1));       %% Trng @ y(10177)  %% only peak and positive lags are assessed - due to symmetry to save time?  AGM
        %% Apply CFAR Detection procedure:  use Slotted Window noise mean estimator
        norm        = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
        y           = y(121:end-50);          %% remove delay and align data and noise level estimate
        norm        = norm(121:end-50);
        % plot((1:length(y))/OSR,y-Kt*norm);pause(0.5);
        % hold on;
        [pk,kp]     = max(y-Kt*norm);       %% Select the peak threshold exceedence
        if(pk>KD)                       %% Use only if y(kp) > KD*norm(kp)
            if(abs(kp-10056)<3)
                det(k) = det(k) + 1;
            end
        end
        %% FAR detection
        [w,l] = xcorr(noise,ref);       %% Cross Correlation Detection of Trng Seq (Noise only)
        
        nmax(k,j) = max(abs(w));
        
        y = abs(w(l>-1));
        norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
        y = y(121:end-50);
        norm = norm(121:end-50);
        % plot((1:length(y))/OSR,y-Kt*norm,'g--');pause(0.5);
        % hold off;
        [pk,kp] = max(y-Kt*norm);
        if(pk>KD)
            far(k) = far(k) + 1;
        end
        
    end
    k = k+1;
end

% May 28, 2017 - error message when running space Norway case, SNR is 7 long, det is 16
% long, error because the number of detections is over the threshold values
% Kt, which in Tim's code sweeps out 16 values
% Error using plot
% Vectors must be the same length.
% Error in TrainingDetectionAM (line 137)
% plot(SNR,det/Ntrials,'o-');grid on;
% plot(SNR,det/Ntrials,'o-');grid on;
figure(11)
plot(Ktmat,det/Ntrials,'o-b');grid on;
hold on
plot(Ktmat,far/Ntrials,'o-r');
hold off
xlabel('Kt threshold')
ylabel('P_D')
title('Detection rate, false alarm rate versus threshold')
ax = gca;
set(0, 'DefaultAxesFontWeight', 'normal', ...
    'DefaultAxesFontSize', 7, ...
    'DefaultAxesFontAngle', 'normal', ... % Not sure the difference here
    'DefaultAxesFontWeight', 'normal', ... % Not sure the difference here
    'DefaultAxesTitleFontWeight', 'normal', ...
    'DefaultAxesTitleFontSizeMultiplier', 1) ;
legend('Blue - detections', 'Red - false alarms');

figure(12)
loglog(far/Ntrials,det/Ntrials)
ylabel('P_D')
xlabel('P_F_A')
grid
title('Detection vs. false alarm rate linear')

figure(13)
plot(far(end:-1:1)/Ntrials,det(end:-1:1)/Ntrials,'r')
ylabel('P_D')
xlabel('P_F_A')
grid
title('Detection vs, false alarm rate log')

figure(14)
plot(rmax,'.k')
hold on
plot(nmax,'.r')
% plot(1:k,rmaxRef,'g')
hold off
ylabel('value')
xlabel('trial')
grid
title('Rmax and nmax trend')

figure(15); plot(rmaxidx,'-')
title('Index location of Rmax peak value (note:  zoom in on dense area)')
grid

figure(16);
histogram(rmaxidx)
title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
grid

figure(17);
histogram(rmaxidx - mean(rmaxidx), 60)
title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
grid

figure(18);
histogram(rmaxidx - mean(rmaxidx), 21, 'BinEdges', -10:10, 'Normalization', 'pdf' )
title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
grid

figure(19)
histogram(rmax)
hold on
histogram(nmax)
hold off
grid
title(['Histogram of Rmax and Nmax peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
xlabel(['Detection peak value, sequence:  ' seq_type]);
legend('Signal','Noise')

if(Ntrials>99000)
    %  save ROC_BestModZadof_Chu_CurveTrngDet_p0dB det far Ntrials SNR;
%     save ROC_SpaceNorway_CurveTrngDet_p0dB det far Ntrials SNR;
    save ROC_BarkerStd_CurveTrngDet_p0dB det far Ntrials SNR;
end
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%