%% Working Group 5 - Training Sequence Analysis
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    07112016
%%  Additions and edits A. Macikunas, 30052017, 01062017, 04062017
%%  Free to use for non-commercial, academic or Standards Body Purposes
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Implements a poor-mans ROC curve estimation for a specific Tarining sequence
%% in a QPSK Modulated message with a BPOSK or other form of training sequence
%% Additions to analysis, new waveforms, updated and new plots, SNR loop
%%   diagnostic data collection for examination of detector statistics AGM May 2017
%% Outputs:
%%     Plots ROC Curves for study, changed to fixed threshold ROC, remove CFAR estimation/threshold
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
clear all;
r2      = sqrt(2);
pi2     = 2*pi;

training_type = 1;
% training_type = menu(' Training sequence to run', 'Barker sequence', 'Zadoff Chu 27 sequence', 'Space Norway 27-bit sequence');
training_type = menu('Select Reference', ...
    'Barker (Jan coder)','13-bit Correlator Barker (Jan coder)','Space Norway 27-bit propsed', 'Zadoff Chu 27-symb', ...
    'Zadoff Chu 32-symb', 'Chu 27-symb', 'Chu 32-symb','Polyphase Barker 27 (TD)','Frank 25', ...
    'Unattributed binary 27 (TD)', 'Unfiltered Zadoff-Chu 27', 'AM 69 phase sequence', ...
    '4 FSK DMR+AGM', '4 FSK other', '4 FSK future', 'Future - various to try');

% load VDE_TrngBarker;            %% sref @ OSR = 8;
if(training_type==1)
    %% Define the proposed Barker-based Training Sequence
    if(1==0)
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];       % Tim's original
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
        TRNG = [1 barker ~barker];
        symbols = [TRNG; TRNG];
        TrngSym = [2 1]*symbols;
        k3 = find(TrngSym==3);
        TrngSym(k3) = 2;
        lTrng = length(TrngSym);
        TRNG = exp(1i*(TrngSym*pi/2+pi/4));     % Tim's original
    else
        %  check if this is still correct, as used in other code - AGM 27 May 2017
        %  ************************************************
        barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
        TRNG = [1 barker ~barker];         %% As currently proposed
        
        trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
        trng_mapped(2:2:2*length(TRNG)) = TRNG;
        
        trng_mapped;
        
        TRNG = qpskMapperJan(trng_mapped);
    end
    
    seq_type = 'Barker QPSK Jan';
elseif(training_type==2)
    fprintf('FUTURE - not supported yet \n');
    
elseif(training_type==3)
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;     % use same symbol mapping as Barker AGM May 2017
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    TRNG = qpskMapperJan(trng_mapped);
    seq_type = 'Space Norway 27 symb';
    
elseif(training_type==4)
    %    load ZadofChu_AmbxxUnFilt; TRNG = bb;  %% "best" 27 symbol Choice so far: Unfiltered
    load ZadofChu_Amb; TRNG = bb;            %
    
    seq_type = 'ZadoffChu 27 symb';
    
elseif(training_type==5)
    
    load ZadofChu32_Ambxy; TRNG = bb;
    seq_type = 'ZadoffChu32-Ambxy';
    
elseif(training_type==6)
    load CHU_27; TRNG = a;  % am - does this exist - need to create it
    seq_type = 'Chu-27';
    
elseif(training_type==7)
    
    load CHU_32; TRNG = a;  % am - does this exist - need to create it
    seq_type = 'Chu-32';
    
elseif(training_type==8)
    
    load polyphase_barker27; TRNG = bb;
    seq_type = 'Polyphase Barker Pi/4-QPSK';
    
elseif(training_type==9)
    
    Frank25 = [0 0 0 0 0 0 1 2 3 4 0 2 4 1 3 0 3 1 4 2 0 4 3 2 1];
    TRNG = exp(1i*pi2*Frank25/5);
    seq_type = 'Frank25';
    
elseif(training_type==10)
    
    [TRNG, ~] = pi4qpskMapper([0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1], 0);
    
    seq_type = 'Unlabelled binary (TD)';
    
elseif(training_type==11)
    
    load ZadofChu_AmbxxUnFilt; TRNG = bb;    %% "best" Choice so far: Unfiltered
    
    seq_type = 'ZadoffChu-27 Unfiltered';
    
elseif(training_type==12)
  
        load AMphase
    TRNG = exp(1i*pi2*phasefn);
    seq_type = 'AM 69 phase sequence';
    
elseif(training_type==13)       % FSK
    if(1==0)
        load txSigref txSig M
        TRNG = txSig.';
        TRNG = txSig;
        shapingf = 0.5;
    else
        data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 1 4 ] - 1; % Arunas 3 not bad, ISL -4.1 dB
        numbersymb  = 27;
        BR          = 19200;
        OSR         = 20;
        OSR         = 8;
        
        fs          = OSR*BR;
        M           = 4;            % Modulation order
        k           = log2(M);      % Bits per symbol
        SymbR       = BR/k;         % symbol rate (Hz)
        % Fs = SymbR * OSR;    % Sample rate (Hz)
        freqsep     = 4 * 1296;  % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X
        
        nsamp       = fs/SymbR;     % Number of samples per symbol
        nsamp       = fs/SymbR/2;     % Number of samples per symbol
        
        data = randi([0 M-1],numbersymb,1)';
        
        txSig = fskmod(data, M,freqsep, nsamp, fs);
        TRNG = txSig;
        
        shapingf = 1;
    end
    
    seq_type = '4FSK DMR+AGM';
    
elseif(training_type==14)
    seq_type = '4FSK like Frank';
    
elseif(training_type==15)
    seq_type = '4FSK future';
    
elseif(training_type==16)
    
    % Files I don't have from Tim
    %     load Max27P2SL_1; ref = pn0;
    %     load Max32P2SL_TAG3; ref = pn0;
    %     load BestZadoff_ChuRev3; ref = bb;
    %     load BestZadoff_ChuRev3; ref = bb;     %% bb Second-BEST - small search range;
    %     load CHU_33;ref = a;                %% CAZAC Sequence
    %     load tempCHUxy;ref = bb;
    %     load tempChu1; ref = bb;              %% Chu BEST;
    % load tempCHUxx1; ref = bb;            %% very Good Intermediate: Unfiltered
end
lTrng = length(TRNG);

%% ---------------------------------------------------
body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));          % what is this?  random pi/4 QPSK? AGM
body(2:2:end)   = body(2:2:end)*exp(1i*pi/4);

OSR                 = 8;
NT                  = 6;         %% Set-up the SRRC Shaping filter
alpha               = 0.3;    %% BW Excess factor
d                   = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,alpha);
H                   = design(d);
hsrrc               = H.numerator;
hsrrc               = hsrrc/sum(hsrrc);
ndelay              = (length(hsrrc)-1)/2;

if(1==0)
    % new code to change to allow use of FSK modulation (from
    % VDE_TrngSeq_Rev1.m) AGM June 2017
    if( training_type < 15  || training_type > 17)
        NN = length(ref);
        % reshape and filter reference function
        iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));  %% Over-sampled SRRC'd Modulation format
        sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
        sref = sref(ndelay+1:end);
    else        % for FSK modulations
        % future modification AGM May 2017 - check for symbol rate scaled SRRC filter corresponds to DMR (shaping factor appears to be 1 for DMR)
        sref = filter(hsrrc,1,[txSig zeros(1,ndelay)]);
        sref = sref(ndelay+1:end);
        NN = length(txSig) / OSR;
    end
end

inBB                = [TRNG body];     %% Test Message at BB

if( training_type < 13 )
    xin                 = kron(inBB,ones(1,OSR));
    ltrain              = lTrng*OSR;
else
    xin             = inBB;     % already resampled - no need to change
    ltrain          = length(TRNG);
end

sig                 = filter(hsrrc,1,[xin zeros(1,ndelay)]);

sig                 = sig(ndelay+1:end);    %% Shaped, modulated Test Waveform
lsig                = length(sig);
ref                 = sig(1:ltrain);        %% Extract the ideal Training F for later detection processing

SNR                 = -6:1:10;               %% Set an SNR (Es/No) range for ROC Curve
SNR                 = -10:2:10;               %% Set an SNR (Es/No) range for ROC Curve

Nsnr                = length(SNR);
Ntrials             = 100000;           %% Need to calcualte this for Statistical Significance
% Ntrials = 100, 30 seconds approx, Ntrials = 10000, 15 minutes, 100000 = hour +?
Ntrials             = 10000;           %% Need to calcualte this for Statistical Significance 15 min
Ntrials             = 1000;           %% Need to calcualte this for Statistical Significance 1 min?
Ntrials             = 100;              %% 30 seconds?
% Ntrials             = 300;              %% 30 seconds?

%% Typically, 90% Confidence intervals
k                   = 1;
Kt                  = 2.9417;                %% Detection threshold: based on Linear Envelope Detector (see Whalen's Text
%% Or use Robertson's curves for Deg of Freedom
KD                  = 0;
initialsize         = 21;
det                 = zeros(1,initialsize);
far                 = det;
snr                 = 10^(SNR(2)/20);
snr = 10^(-3/20);
% snr = 10^(0/20);

Ktmat               = 2:(1/3):7.25;

%% Create reference level and peak location - noiseless
x                   = [zeros(1,lsig) sig zeros(1,lsig)];     %% create message WF without noise for pk and std dev estimate
[r,l]               = xcorr(ref,x);       %% Cross Correlation Detection of Rx signal with itself
rstdRef             = std(r);
[rmaxRef, mxidxRef] = max(abs(r));
clear x r l
CFAR                = 0;        % if CFAR = 1, use CFAR method, else fixed threshold

% for Kt = 2:(1/3):7.25       %% Vary the detection threshold to sweep out PD v. PFA curve
for m = 1:length(SNR)       %% Vary the detection threshold to sweep out PD v. PFA curve
    snr             = 10^(SNR(m)/20);
    det(k) = 0;
    far(k) = 0;
    for j=1:Ntrials
        %         noise = cawgn(3*lsig)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No
        noise       = wgn(1,3*lsig,0)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No Modifed AGM used Matlab White Gaussian Noise, normalized power, lenght = arg
        
        x           = noise + [zeros(1,lsig) sig zeros(1,lsig)];     %% place message WF in AWGN
        
        [r,l]       = xcorr(x,ref);       %% Cross Correlation Detection of Trng Seq
        
        [mx, mxidx] = max(abs(r));
        rmax(m,j)   = mx;
        rmaxidx(m,j)= mxidx - 40704;            % empirical center index
        %         rmaxidx(m,j) = mxidx - mxidxRef;
        
        y = abs(r(l>-1));       %% Trng @ y(10177)  %% only peak and positive lags are assessed - due to symmetry to save time?  AGM
        if(CFAR==1)
            %% Apply CFAR Detection procedure:  use Slotted Window noise mean estimator
            norm        = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
            y           = y(121:end-50);          %% remove delay and align data and noise level estimate
            norm        = norm(121:end-50);
            % plot((1:length(y))/OSR,y-Kt*norm);pause(0.5);
            % hold on;
            [pk,kp]     = max(y-Kt*norm);       %% Select the peak threshold exceedence
            if(pk>KD)                       %% Use only if y(kp) > KD*norm(kp)
                if(abs(kp-10056)<3)
                    det(k) = det(k) + 1;
                end
            end
        end
        %% FAR detection
        [w,l] = xcorr(noise,ref);       %% Cross Correlation Detection of Trng Seq (Noise only)
        
        [mx, mxidx] = max(abs(w));
        nmax(m,j)   = mx;
        nmaxidx(m,j)= mxidx - 40703;            % empirical center index
        if(CFAR==1)
            y = abs(w(l>-1));
            norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
            y = y(121:end-50);
            norm = norm(121:end-50);
            % plot((1:length(y))/OSR,y-Kt*norm,'g--');pause(0.5);
            % hold off;
            [pk,kp] = max(y-Kt*norm);
            if(pk>KD)
                far(k) = far(k) + 1;
            end
        end
        
    end
    k = k+1;
end
% kmax = k - 1;
kmax            = length(SNR);
pk_max          = max([ max(max(rmax)) max(max(nmax))]);
pk_min          = min([ min(min(rmax)) min(min(nmax))]);
% highSNRmidpoint = max(nmax(kmax,:)) - min(rmax(kmax,:));
highSNRmidpoint = ( mean(rmax(kmax,:)) - mean(nmax(kmax,:)) ) /2;
numsteps        = 81;

numcenterpoints = fix( numsteps/1.5 );
numtopbottom    = fix( (numsteps - numcenterpoints) / 2);
topbottomscale  = 3/4;

jthresh(1:numtopbottom) = (pk_min + (1:numtopbottom)*(highSNRmidpoint-pk_min)*topbottomscale/(numtopbottom)) ;
jthresh((numtopbottom+1):(numsteps-numtopbottom)) = (1:numcenterpoints+1)*(highSNRmidpoint-pk_min)/(numtopbottom);
% jthresh((numtopbottom+1):(numsteps-numtopbottom)) = -99;

% jthresh( (numsteps-numtopbottom+1):numsteps) = (highSNRmidpoint+pk_min + (1:numtopbottom)*(highSNRmidpoint+pk_min)/(numtopbottom)) ;
jthresh( (numsteps-numtopbottom+1):numsteps) = (pk_max - (numtopbottom-1:-1:0)*(pk_max-highSNRmidpoint)*topbottomscale/(numtopbottom)) ;

if(CFAR==0)
    det                 = zeros(kmax,numsteps);
    far                 = det;
    
    for j = 1:kmax
        for k = 1:numsteps
            %             jthresh = pk_min + (k-1)*(pk_max-pk_min)/(numsteps);
            %             if(j==1)
            %             threshmat(k) = jthresh;
            %             end
            [a, b] =  find( rmax(j,:)>jthresh(k));
            det(j,k) = det(j,k) + length( a );
            [a, b] =   find( nmax(j,:)>jthresh(k));
            far(j,k) = far(j,k) + length( a );
            
        end
    end
end
threshmat = jthresh;
seq_type
median(rmaxidx,2)', std(rmaxidx')

% May 28, 2017 - error message when running space Norway case, SNR is 7 long, det is 16
% long, error because the number of detections is over the threshold values
% Kt, which in Tim's code sweeps out 16 values
% Error using plot
% Vectors must be the same length.
% Error in TrainingDetectionAM (line 137)
% plot(SNR,det/Ntrials,'o-');grid on;
% plot(SNR,det/Ntrials,'o-');grid on;

ax = gca;
set(0, 'DefaultAxesFontWeight', 'normal', ...
    'DefaultAxesFontSize', 12, ...
    'DefaultAxesFontAngle', 'normal', ... % Not sure the difference here
    'DefaultAxesFontWeight', 'normal', ... % Not sure the difference here
    'DefaultAxesTitleFontWeight', 'normal', ...
    'DefaultAxesTitleFontSizeMultiplier', 1) ;

figure(11)
% plot(Ktmat,det/Ntrials,'o-b');grid on;
plot(threshmat,det/Ntrials,'o-b');grid on;
hold on
% plot(Ktmat,far/Ntrials,'o-r');
plot(threshmat,far/Ntrials,'o-r');

hold off
xlabel('Kt threshold')
ylabel('P_D')
title('Detection rate, false alarm rate versus threshold')
legend('Blue - detections', 'Red - false alarms');

figure(12)
loglog(far/Ntrials,det/Ntrials)
ylabel('P_D')
xlabel('P_F_A')
grid
title('Detection vs. false alarm rate linear')

figure(13)
plot(far(end:-1:1)'/Ntrials,det(end:-1:1)'/Ntrials,':*k')
ylabel('P_D')
xlabel('P_F_A')
grid
title('Detection vs, false alarm rate log')

figure(14)
plot(SNR, rmax,'.k')
hold on
plot(SNR, nmax,'.r')
% plot(1:k,rmaxRef,'g')
hold off
ylabel('value')
xlabel('SNR (dB)')
grid
title(['Rmax and nmax trend, seq type ' seq_type])

figure(15); plot(rmaxidx,'-')
title('Index location of Rmax peak value (note:  zoom in on dense area)')
grid

figure(16);
histogram(rmaxidx(end,:))
title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
grid

figure(17);
histogram(rmaxidx - mean(rmaxidx), 60)
% title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
title(['Histogram of Index of peak values, all SNRs = ' seq_type ])

grid

figure(18);
histogram(rmaxidx - mean(rmaxidx), 21, 'BinEdges', -10:10, 'Normalization', 'pdf' )
title(['Histogram of Index of peak values, all SNR, zoomed in, ' seq_type ])
xlabel(['Offset from nominal center (samples), ' num2str(OSR) ' samples per symbol']);
grid

figure(19)
histogram(rmax)
hold on
histogram(nmax)
hold off
grid
title(['Histogram of Rmax and Nmax peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
xlabel(['Detection peak value, sequence:  ' seq_type]);
legend('Signal','Noise')

if(Ntrials>99000)
    %  save ROC_BestModZadof_Chu_CurveTrngDet_p0dB det far Ntrials SNR;
    %     save ROC_SpaceNorway_CurveTrngDet_p0dB det far Ntrials SNR;
    save ROC_BarkerStd_CurveTrngDet_p0dB det far Ntrials SNR;
end
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%