%% Working Group 5 - Training Sequence Analysis
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    07112016
%%  Additions and edits A. Macikunas, 30052017, 01062017, 04062017
%%  Free to use for non-commercial, academic or Standards Body Purposes
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Implements a poor-mans ROC curve estimation for a specific Tarining sequence
%% in a QPSK Modulated message with a BPOSK or other form of training sequence
%% Additions to analysis, new waveforms, updated and new plots, SNR loop
%%   diagnostic data collection for examination of detector statistics AGM May 2017
%% Outputs:
%%     Plots ROC Curves for study, changed to fixed threshold ROC, remove CFAR estimation/threshold
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
clear all;
r2      = sqrt(2);

Ntrials             = 100000;           %% Need to calcualte this for Statistical Significance
% Ntrials = 100, 30 seconds approx, Ntrials = 10000, 15 minutes, 100000 = hour +?
Ntrials             = 10000;           %% Need to calcualte this for Statistical Significance 15 min
Ntrials             = 1000;           %% Need to calcualte this for Statistical Significance 1 min?
Ntrials             = 100;              %% 30 seconds?
Ntrials             = 300;              %% 30 seconds?

cc          = ['b' 'g' 'c' 'm' 'r' 'k'];
BR          = 19200;     % QPSK bit rate
% OSR         = 20;
OSR                 = 8;

fs          = OSR*BR;
NT          = 6;                % shaping filter parameter
pi2         = 2*pi;
shapingf    = 0.3;              % default SRRC shaping factor
% shapingf = .25;

%% for FSK modulation
M           = 4;                % Modulation order for FSK ( = 4 FSK )
k           = log2(M);          % Bits per symbol
SymbR       = BR/k;             % symbol rate (Hz)
% Fs = SymbR * OSR;         % Sample rate (Hz)

% nsamp       = fs/SymbR;       % Number of samples per symbol
nsamp       = fs/SymbR/2;       % Number of samples per symbol

%         freqsep = SymbR/8;  % Frequency separation (Hz)
% freqsep     = 2 * 1296;       % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X
freqsep     = 4 * 1296;         % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X

training_type = [];
[shapingf_override, training_type, seq_type, data, ref, refshort, reflen, txSig, txSiglen] = ...
    select_sequence( M,freqsep, nsamp, fs, training_type);

if( ~isempty(shapingf_override) )
    shapingf = shapingf_override;
end

seq_type = [ seq_type ', shaping = ' num2str(shapingf)];

%% ---------------------------------------------------
% body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));          %  random pi/4 QPSK? AGM

% body(2:2:end)   = body(2:2:end)*exp(1i*pi/4);                           % this is not pi/4 QPSK - deleted, unnecessary

% OSR                 = 8;
% NT                  = 6;                %% Set-up the SRRC Shaping filter
alpha               = shapingf;         %% BW Excess factor
d                   = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,alpha);
H                   = design(d);
hsrrc               = H.numerator;
hsrrc               = hsrrc/sum(hsrrc);
ndelay              = (length(hsrrc)-1)/2;

if(1==0)
    % new code to change to allow use of FSK modulation (from
    % VDE_TrngSeq_Rev1.m) AGM June 2017
    if( training_type < 15  || training_type > 17)
        NN = length(ref);
        % reshape and filter reference function
        iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));  %% Over-sampled SRRC'd Modulation format
        sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
        sref = sref(ndelay+1:end);
    else        % for FSK modulations
        % future modification AGM May 2017 - check for symbol rate scaled SRRC filter corresponds to DMR (shaping factor appears to be 1 for DMR)
        sref = filter(hsrrc,1,[txSig zeros(1,ndelay)]);
        sref = sref(ndelay+1:end);
        NN = length(txSig) / OSR;
    end
end

% add FSK pre-modulated signal here?  AGM June 2017

if( training_type < 15  || training_type > 17)
    body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));          %  random pi/4 QPSK? AGM
    inBB            = [ref body];     %% Test Message at BB
    
    xin             = kron(inBB,ones(1,OSR));
    lTrng           = length(ref);
    ltrain          = lTrng*OSR;
else
    % hard code sequence from DMR+AGM customization
%     data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
% variable data should be supplied by routine select_sequence
    lTrng           = length(data);
    body            = floor(rand(1,1245)*4);          %  random 4FSK symbols AGM
    data            = [data body];
    xin             = fskmod(data, M,freqsep, nsamp, fs);
    ltrain          = lTrng*OSR;
%     xin             = inBB;     % already resampled - no need to change
%     ltrain          = length(ref);

end

sig                 = filter(hsrrc,1,[xin zeros(1,ndelay)]);

sig                 = sig(ndelay+1:end);    %% Shaped, modulated Test Waveform
lsig                = length(sig);
ref                 = sig(1:ltrain);        %% Extract the ideal Training F for later detection processing

SNR                 = -6:1:10;               %% Set an SNR (Es/No) range for ROC Curve
SNR                 = -10:2:0;               %% Set an SNR (Es/No) range for ROC Curve
% SNR                 = -9:3:0;               %% Set an SNR (Es/No) range for ROC Curve

Nsnr                = length(SNR);

%% Typically, 90% Confidence intervals
k                   = 1;
Kt                  = 2.9417;                %% Detection threshold: based on Linear Envelope Detector (see Whalen's Text
%% Or use Robertson's curves for Deg of Freedom
KD                  = 0;
initialsize         = 21;
det                 = zeros(1,initialsize);
far                 = det;
snr                 = 10^(SNR(2)/20);
snr = 10^(-3/20);
% snr = 10^(0/20);

Ktmat               = 2:(1/3):7.25;

%% Create reference level and peak location - noiseless
x                   = [zeros(1,lsig) sig zeros(1,lsig)];     %% create message WF without noise for pk and std dev estimate
[r,l]               = xcorr(ref,x);       %% Cross Correlation Detection of Rx signal with itself
rstdRef             = std(r);
[rmaxRef, mxidxRef] = max(abs(r));
clear x r l
CFAR                = 0;        % if CFAR = 1, use CFAR method, else fixed threshold

% for Kt = 2:(1/3):7.25       %% Vary the detection threshold to sweep out PD v. PFA curve
for m = 1:length(SNR)       %% Vary the detection threshold to sweep out PD v. PFA curve
    snr             = 10^(SNR(m)/20);
    det(k) = 0;
    far(k) = 0;
    % m is outer loop - over SNR, threshold (Tim D original), Doppler (AGM)
    for j=1:Ntrials
        %         noise = cawgn(3*lsig)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No
        noise       = wgn(1,3*lsig,0)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No Modifed AGM used Matlab White Gaussian Noise, normalized power, lenght = arg
        
        x           = noise + [zeros(1,lsig) sig zeros(1,lsig)];     %% place message WF in AWGN
        
        [r,l]       = xcorr(x,ref);       %% Cross Correlation Detection of Trng Seq
        
        [mx, mxidx] = max(abs(r));
        rmax(m,j)   = mx;
        rmaxidx(m,j)= mxidx - 40704;            % empirical center index
        %         rmaxidx(m,j) = mxidx - mxidxRef;
        
        y = abs(r(l>-1));       %% Trng @ y(10177)  %% only peak and positive lags are assessed - due to symmetry to save time?  AGM
        if(CFAR==1)
            %% Apply CFAR Detection procedure:  use Slotted Window noise mean estimator
            norm        = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
            y           = y(121:end-50);          %% remove delay and align data and noise level estimate
            norm        = norm(121:end-50);
            % plot((1:length(y))/OSR,y-Kt*norm);pause(0.5);
            % hold on;
            [pk,kp]     = max(y-Kt*norm);       %% Select the peak threshold exceedence
            if(pk>KD)                       %% Use only if y(kp) > KD*norm(kp)
                if(abs(kp-10056)<3)
                    det(k) = det(k) + 1;
                end
            end
        end
        %% FAR detection
        [w,l] = xcorr(noise,ref);       %% Cross Correlation Detection of Trng Seq (Noise only)
        
        [mx, mxidx] = max(abs(w));
        nmax(m,j)   = mx;
        nmaxidx(m,j)= mxidx - 40704;            % empirical center index
        if(CFAR==1)
            y = abs(w(l>-1));
            norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
            y = y(121:end-50);
            norm = norm(121:end-50);
            % plot((1:length(y))/OSR,y-Kt*norm,'g--');pause(0.5);
            % hold off;
            [pk,kp] = max(y-Kt*norm);
            if(pk>KD)
                far(k) = far(k) + 1;
            end
        end
        
    end
    k = k+1;
end
% kmax = k - 1;
kmax            = length(SNR);
pk_max          = max([ max(max(rmax)) max(max(nmax))]);
pk_min          = min([ min(min(rmax)) min(min(nmax))]);
% highSNRmidpoint = max(nmax(kmax,:)) - min(rmax(kmax,:));
highSNRmidpoint = ( mean(rmax(kmax,:)) + mean(nmax(kmax,:)) ) /2;
% highSNRmidpoint = ( mean(rmax(kmax,:))

numsteps        = 81;
% numsteps        = 21;

numcenterpoints = fix( numsteps/1.5 );
numtopbottom    = fix( (numsteps - numcenterpoints) / 2);
topbottomscale  = 0.6;      % empirically chosen to stretch the middle section of threshold

jthresh(1:numtopbottom) = (pk_min + (1:numtopbottom)*(highSNRmidpoint-pk_min)*topbottomscale/(numtopbottom)) ;
jthresh((numtopbottom+1):(numsteps-numtopbottom)) = (1:numcenterpoints+1)*(highSNRmidpoint-jthresh(numtopbottom))/(numtopbottom) + jthresh(numtopbottom);

% jthresh((numtopbottom+1):(numsteps-numtopbottom)) = -99;

% jthresh( (numsteps-numtopbottom+1):numsteps) = (highSNRmidpoint+pk_min + (1:numtopbottom)*(highSNRmidpoint+pk_min)/(numtopbottom)) ;
jthresh( (numsteps-numtopbottom+1):numsteps) = (pk_max - (numtopbottom-1:-1:0)*(pk_max-highSNRmidpoint)*topbottomscale/(numtopbottom)) ;

[tmp,midjthreshidx]     = min(abs(jthresh-highSNRmidpoint));        % middle threshold, signal to noise high SNR

%  loop to run detection statistics and false alarms as a function of
%  threshold, also to consider detections based on correct peak location
if(CFAR==0)
    det                 = zeros(kmax,numsteps);
    far                 = det;
    detpeak             = det;
    detpeak3            = det;
    detpeak5            = det;
    detpeak80           = det;
    rmsSymError         = det;
    % kmax is out loop, SNR, Doppler, etc.
    % numsteps is the number of divisions in the detetion criteria (threshold)
    for m = 1:kmax
        for j = 1:numsteps
            %             jthresh = pk_min + (j-1)*(pk_max-pk_min)/(numsteps);
            %             if(m==1)
            %             threshmat(j) = jthresh;
            %             end
            [a, b]      =  find( rmax(m,:) > jthresh(j));
            
            det(m,j)    = det(m,j) + length( a );
            
            % detections at locations in b
            
            [a, b1]      = find( abs(rmaxidx(m,b)) < 1);
            detpeak(m,j)   = detpeak(m,j) + length( a );
            
            [a, b1]      = find( abs(rmaxidx(m,b)) < 2);
            detpeak3(m,j)   = detpeak3(m,j) + length( a );
            
            [a, b1]      = find( abs(rmaxidx(m,b)) < 3);
            detpeak5(m,j)        = detpeak5(m,j) + length( a );
            
            [a, b1]      = find( abs(rmaxidx(m,b)) < 41);
            detpeak80(m,j)        = detpeak80(m,j) + length( a );
            % on calculate rms error when there are valid peaks within 40
            % samples of target location, and at least 3 points
            if(length(b1)>3)
                rmsSymError(m,j)      = std(rmaxidx(m,b1));
            else
                rmsSymError(m,j)      = NaN;
            end
            [a, b] =   find( nmax(m,:)>jthresh(j));
            far(m,j) = far(m,j) + length( a );            
        end
    end
end
rmsSymError = rmsSymError / OSR;

threshmat = jthresh;
fprintf('%s \n', seq_type)
for k = 1:kmax
    fprintf('SNR %3.0f (dB), mid S+N-N threshold, (+/- 1 sample) PD %.2f, PFA %.2f (%%) \n', ...
        SNR(k), detpeak3(k,midjthreshidx)/Ntrials*100, far(k,midjthreshidx)/Ntrials*100);
    fprintf('   ----> symbol peak standard deviation %.2f (Symb), median peak location %.0f (samples)\n', ...
        rmsSymError(k,midjthreshidx), median(rmaxidx(k,:)));
end
    fprintf('All SNRs, %.0f points, mid S+N-N threshold, (+/- 1 sample) PD %.2f, PFA %.2f (%%) \n', ...
        prod(size(rmsSymError)), sum(detpeak3(:,midjthreshidx))/Ntrials/kmax*100, sum(far(:,midjthreshidx))/Ntrials/kmax*100);
    fprintf('   ----> peak correlated standard deviation %.2f (Symb), median peak location %.0f (samples)\n', ...
        std(rmsSymError(:,midjthreshidx)), median(median(rmaxidx)));
% median(rmaxidx,2)', std(rmaxidx')/OSR       % median value, rms error in estimate in symbols
% rmsSymError;

% May 28, 2017 - error message when running space Norway case, SNR is 7 long, det is 16
% long, error because the number of detections is over the threshold values
% Kt, which in Tim's code sweeps out 16 values
% Error using plot
% Vectors must be the same length.
% Error in TrainingDetectionAM (line 137)
% plot(SNR,det/Ntrials,'o-');grid on;
% plot(SNR,det/Ntrials,'o-');grid on;

ax = gca;
set(0, 'DefaultAxesFontWeight', 'normal', ...
    'DefaultAxesFontSize', 12, ...
    'DefaultAxesFontAngle', 'normal', ... % Not sure the difference here
    'DefaultAxesFontWeight', 'normal', ... % Not sure the difference here
    'DefaultAxesTitleFontWeight', 'normal', ...
    'DefaultAxesTitleFontSizeMultiplier', 1) ;

figure(11)
% plot(Ktmat,det/Ntrials,'o-b');grid on;        % Tim Dyson's version
plot(threshmat,det/Ntrials,'o-b');grid on;
hold on
% plot(Ktmat,far/Ntrials,'o-r');
plot(threshmat,far/Ntrials,'o-r');

hold off
xlabel('Detection threshold')
ylabel('P_D')
title('Detection rate, false alarm rate versus threshold')
legend('Blue - detections', 'Red - false alarms');

figure(12)
loglog(far/Ntrials,det/Ntrials)
ylabel('P_D')
xlabel('P_F_A')
grid
title('Detection vs. false alarm rate log')

figure(13)
plot(far'/Ntrials,det'/Ntrials,':+')
ylabel('P_D')
xlabel('P_F_A')
grid
title('All Detections vs false alarm rate')

figure(33)
plot(far'/Ntrials,detpeak5'/Ntrials,':+')
ylabel('P_D')
xlabel('P_F_A')
grid
title('Low Timing Error Detection vs false alarm rate - linear')

figure(34)
plot(far'/Ntrials,detpeak3'/Ntrials,':+')
ylabel('P_D')
xlabel('P_F_A')
grid
title('Very Low Timing Error Detection vs false alarm rate - linear')

figure(14)
plot(SNR, rmax,'.k')
hold on
plot(SNR, nmax,'.r')
% plot(1:k,rmaxRef,'g')
hold off
ylabel('value')
xlabel('SNR (dB)')
grid
title(['Rmax and nmax trend, seq type ' seq_type])

figure(15); plot(rmaxidx,'-')
title('Index location of Rmax peak value (note:  zoom in on dense area)')
grid

figure(16);
histogram(rmaxidx(end,:))
title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
grid

figure(17);
histogram(rmaxidx - mean(rmaxidx), 60)
% title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
title(['Histogram of Index of peak values, all SNRs = ' seq_type ])
grid

figure(18);
histogram(rmaxidx - mean(rmaxidx), 21, 'BinEdges', -10:10, 'Normalization', 'pdf' )
title(['Histogram of Index of peak values, all SNR, zoomed in, ' seq_type ])
xlabel(['Offset from nominal center (samples), ' num2str(OSR) ' samples per symbol']);
grid

figure(19)
histogram(rmax)
hold on
histogram(nmax)
hold off
grid
title(['Histogram of Rmax and Nmax peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
xlabel(['Detection peak value, sequence:  ' seq_type]);
legend('Signal','Noise')

figure(30); plot(rmax, rmaxidx,'.:')
title(' rmax peak versus rmax peak index (error)')

if(Ntrials>99000)
    %  save ROC_BestModZadof_Chu_CurveTrngDet_p0dB det far Ntrials SNR;
    %  save ROC_SpaceNorway_CurveTrngDet_p0dB det far Ntrials SNR;
    save ROC_BarkerStd_CurveTrngDet_p0dB det far Ntrials SNR;
end
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%