%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    08172016
%%  Free to use for non-commercial, academic or Standards Body use
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%%  Brute force optimizer for search to find "Best" Peak/Sidelobe ratio
%%  constant envelope Polyphase Traing Sequence.  Optimization is over the
%%  entire Cross-Ambiguity function for a LEO Satellite Receiver in the
%%  international Maritime Mobile band.
%%
%%  Max Doppler + LO Uncertainty for +/-65 deg elevation equates to
%%  ~ +/- 5200 Hz at 162 MHz so this is the minimum search Doppler.
%%  Optimization is with respect to the linear cross-correlation/ambiguity function
%%  which is the relavant method for the orbital receiver due to the operating
%%  mode needed in this CCI limited channel as opposed to Circular Correlation
%%  used to characterize CAZAC sequences, which assumes pre-alerted search.
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% -----------------------------------------------------------------------
%function VDE_TrngSeq_Rev(BARKER)
clear all;
training_type = menu('Select Reference','2092 Barker sequence (from Jan)', 'Barker created (TimD rotated QPSK)', ...
    'Barker (Jan coder)','13-bit Correlator Barker (Jan coder)','Space Norway 27-bit propsed', 'Zadoff Chu 27-symb', ...
    'Zadoff Chu 32-symb', 'Chu 27-symb', 'Chu 32-symb','Polyphase Barker 27 (TD)','Frank 25', ...
    'Unattributed binary 27 (TD)', 'Unfiltered Zadoff-Chu 27', 'AM 69 phase sequence', 'Future - various to try');

% training_type = 0;
% close all;
format compact;
format short

cc = ['b' 'g' 'c' 'm' 'r' 'k'];
BR = 19200;
OSR = 10;
OSR = 20;

fs = OSR*BR;
NT = 6;
pi2 = 2*pi;
shapingf = 0.3;
% shapingf = .25;

if(training_type==1)
    
    % Copy and paste of the VDES seq generated by my code - pi/4-QPSK:
    ref = [0.70711 + 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, 0.70711 + 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, -1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, -1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i];
    seq_type = 'Barker QPSK 2092 Jan';
    
elseif(training_type==2)
    % JanS: Tim's original code - pi/4 rotated BPSK, this is not pi/4-QPSK!!!!!
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker]          %% As currently proposed
    symbols = [TRNG; TRNG];             %% Set-up for phase map
    TrngSym = [2 1]*symbols;
    TrngSym(TrngSym==3) = 2;
    ref = exp(1i*(TrngSym*pi/2+pi/4));  %% pi/4 rotated BPSK
    %     iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));
    %     sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
    %     sref = sref(ndelay+1:end);
    seq_type = 'Barker pi/r rotated QPSK';
    
elseif(training_type==3)
    
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker];         %% As currently proposed
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG)) = TRNG;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    seq_type = 'Barker QPSK Jan';
    
elseif(training_type==4)
    
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker];         %% As currently proposed
    TRNGshort = [barker];
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG)) = TRNG;
    trngshort_mapped(1:2:(2*length(TRNGshort)-1)) = TRNGshort;
    trngshort_mapped(2:2:2*length(TRNGshort)) = TRNGshort;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    [refshort] = qpskMapperJan(trngshort_mapped);
    seq_type = '13-bit only Barker QPSK Jan';
    
elseif(training_type==5)
    
    % New Space Norway Training Sequence
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 1 0 1 1 ];  % 1 bit altered - AGM to see impact
    
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    seq_type = 'Space Norway 27';
    
elseif(training_type==6)
    
    zad = menu('Which ZadofChu 27-bit sequence','Default', 'Unfiltered');
    
    if(zad==1)
        load ZadofChu_Amb; ref = bb;            %
    else
        load ZadofChu_AmbxxUnFilt; ref = bb;    %% "best" Choice so far: Unfiltered
    end
    seq_type = 'ZadoffChu-27-symb';
    
elseif(training_type==7)
    
    load ZadofChu32_Ambxy; ref = bb;
    seq_type = 'ZadoffChu32-Ambxy';
    
    
elseif(training_type==8)
    load CHU_27; ref = a;  % am - does this exist - need to create it
    seq_type = 'Chu-27';
    
elseif(training_type==9)
    
    load CHU_32; ref = a;  % am - does this exist - need to create it
    seq_type = 'Chu-32';
    
elseif(training_type==10)
    
    load polyphase_barker27; ref = bb;
    seq_type = 'Random Search Pi\4-QPSK';
    
elseif(training_type==11)
    
    Frank25 = [0 0 0 0 0 0 1 2 3 4 0 2 4 1 3 0 3 1 4 2 0 4 3 2 1];
    ref = exp(1i*pi2*Frank25/5);
    seq_type = 'Frank25';
    
elseif(training_type==12)
    
    [ref, ~] = pi4qpskMapper([0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1], 0);
    
    seq_type = 'Unlabelled binary (TD)';
    
elseif(training_type==13)
    
    load ZadofChu_AmbxxUnFilt; ref = bb;    %% "best" Choice so far: Unfiltered
    
    seq_type = 'ZadoffChu-27 Unfiltered';
    
elseif(training_type==14)
    if(1==0)
    phasefn = pi/180* [143.999883964013,157.998625492865,80.6601383580463,125.505473402409,-47.7831995128548,-38.6672199679412,-155.433191098205,-172.904511808408,-73.0862794751650,-59.9186651412132,161.070957430118,142.842665670704,172.339404074901,47.3647678495325,-4.22531712412601,-57.6785977737499,-177.106307110185,-115.317826299311,-76.4249872526181,6.95616231697665,168.515635982310,130.735749482123,88.4216148241380,-9.88227000210407,-172.728430511086,-112.500295778172,-41.3682672125228,59.3779378548736,179.987676498986,52.4693247207682,-59.9010382147719,-131.336241637577,-156.935563951675,18.7964507151681,178.958252560295,59.9624047016509,-69.8217124160105,-145.171386886692,-136.651141316970,-46.1722826285207,84.3698167853535,171.780529683638,136.801341571765,-18.4056301772432,-179.991044342716,-20.7104495328089,140.640254603021,174.783046724681,61.7102983929129,-97.7770789359145,-166.119694584109,-84.7679220783924,72.3985742161437,160.137571571137,93.8285141569285,-64.9949252277021,-159.641713782967,-90.9131014605413,75.1464427376828,164.658256243350,75.9817911374775,-101.901535392152,-172.775415552544,-47.2733157442497,143.504431484207,179.296300532375,2.04059699920865,-16.6553996772684,-177.411658954999];
    else
    load AMphase
    end
    ref = exp(1i*pi2*phasefn);
    seq_type = 'AM 69 phase sequence';
    
elseif(training_type==15)
    
    % Files I don't have from Tim
    %     load Max27P2SL_1; ref = pn0;
    %     load Max32P2SL_TAG3; ref = pn0;
    %     load BestZadoff_ChuRev3; ref = bb;
    %       load BestZadoff_ChuRev3; ref = bb;     %% bb Second-BEST - small search range;
    %     load CHU_33;ref = a;                %% CAZAC Sequence
    %     load tempCHUxy;ref = bb;
    %      load tempChu1; ref = bb;              %% Chu BEST;
    % load tempCHUxx1; ref = bb;            %% very Good Intermediate: Unfiltered
    
end

seq_type = [ seq_type ' shaping ' num2str(shapingf)];
NN = length(ref);

% shaping factor defined above, note a value of  0.3
d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,shapingf);  % rolloff factor is nom 0.3 AGM
% d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,2-1);  % rolloff factor is 0.3 for Beta = 1.3 - 1 AGM

% filter shaping is 2 * NT = 13 taps
% Jan's original
% d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2, 0.3); % Jan: Changed from 1.3 - 1 to 1
% comment says to change to 1, but it is changed back to 0.3 in code provided on Sept 2016

H = design(d);
hsrrc = H.numerator;
hsrrc = hsrrc/sum(hsrrc);
ndelay = (length(hsrrc)-1)/2;
% save ShapingFIR hsrrc ndelay OSR;


%% Basic auto-correlation ----------------------------------------------

% reshape and filter reference function
iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));  %% Over-sampled SRRC'd Modulation format
sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
sref = sref(ndelay+1:end);
% sref = kron(ref,ones(1,OSR));

% delf = 1500;
% sref2 = sref;
% sref = sref.*exp(pi2*1j*delf/fs*(0:(length(sref)-1)));  % implement a frequency shift of reference waveform
% [r,l] = xcorr(sref,sref2);               %% auto-correlation for reference
if(training_type==4)
    irefshort = reshape([refshort.' zeros(length(refshort),OSR-1)].',1,OSR*length(refshort));  %% Over-sampled SRRC'd Modulation format
    srefshort = filter(hsrrc,1,[irefshort zeros(1,ndelay)]);
    srefshort = srefshort(ndelay+1:end);
    
    [r,l] = xcorr(srefshort,sref);               %% auto-correlation for reference
else
    [r,l] = xcorr(sref,sref);               %% auto-correlation for reference
end

[ pk_corr pkcidx ] = max(r);
% find mainlobe width 1, 3 and 6 dB down, max +/- 4 symbol search range
% [ sidelobe_pwr T1dBsidelobidx] = find(abs(r) >= abs(pk_corr)*10.^(-1/20));
% [ sidelobe_pwr T3dBsidelobidx] = find(abs(r) >= abs(pk_corr)*10.^(-3/20));
% [ sidelobe_pwr T6dBsidelobidx] = find(abs(r) >= abs(pk_corr)*10.^(-6/20));

% don't use above for ambiguities below - offset in indices
pk0                 = max(abs(r));

%% single Doppler ambiguity analysis
[PKS,LOCS,W,P]      = findpeaks( abs(r)/abs(max(r)), l);

figure(29)
findpeaks(abs(r)/max(abs(r)),l,'Annotate','extents')
%%eliminate non-prominent peaks (not obviously a peak, empirical threshold -35 dB), and those peaks
% below -30 dB
pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);
lenpeak     = length(PKS)
[a1, b1]    = find(pks_dB < -30);
PKS(b1)     = [];
LOCS(b1)    = [];
W(b1)       = [];
P(b1)       = [];
lenpeak     = length(PKS)

pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);

[a2, b2] = find(prom_pks_dB < -35);
PKS(b2)     = [];
LOCS(b2)    = [];
W(b2)       = [];
P(b2)       = [];
lenpeak     = length(PKS)

[a3, peaksubloc]    = max(PKS);
pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);

% alternate sidelobe measurements based on identified peaks of single
% Doppler cross-correlation function
maindB      = pks_dB(peaksubloc);
sidePKS     = PKS;
sidePKS(peaksubloc)     = [];
% maindB      = max(pks_dB)
sidePKSdB   = 20*log10( sidePKS);

totaldB       = 10*log10(sum(PKS.^2));
peakdB        = 20*log10(max(sidePKS));
% intsidedB     = 10*log10( sum(PKS.^2) - PKS(peaksubloc).^2 )
intsidedB     = 10*log10( sum(sidePKS.^2)  );

% diffPwr         = totalPwr - sidelobePwr;
% totalPwrdB      = 10*log10(totalPwr);
% sidelobePwrdB   = 10*log10(sidelobePwr);

figure(28)
histogram(sidePKSdB,7,'BinLimits',[-35 0])
axis([-35 0 0 25])
grid
title('Histogram of distribution of peaks in different attentuation ranges (by number of peaks)')

figure(12); 
plot(l,20*log10(abs(r/pk0)), '.');
hold on; 
plot(LOCS, pks_dB, '+r');
%plot(LOCS, prom_pks_dB, '*g');
plot(LOCS, 20*log10(P), '*g');
title('Peaks of cross-correlation function');
xlabel('lag offset')
ylabel('Response (dB)');
hold off; 
grid

%% Now compute the Cross-Ambiguity over the Satellite Doppler Extremes
freqstep        = 10;                   % frequency steps - in Hz
df              = -5000:freqstep:5000;
lf              = length(df);
DR              = zeros(lf,length(r));
for m=1:lf
    dref        = sref.*exp(1i*pi*2*df(m)/fs*(0:length(sref)-1));
    [dr,l]      = xcorr(dref,sref);
    DR(m,:)     = abs(dr);                                                          % does not need to be absolute value here
end

%% Window for statistics purposes
PK              = max(max(DR));
dfmax1          = 5000;                 % maximum Doppler plus tolerance, low LEO AGM
% ************ fix - center freq indices not right when I change this to 4250 Hz - can't
% find why?
tsmax1          = floor(max(l)/OSR*0.9);
ddf1            = find(abs(df)<=dfmax1);
% ddf             = ddf(1:2:end);
dl1             = find(abs(l)/OSR<=tsmax1);
% dl1     = l;
dlcent          = floor(length(dl1)/2)+1;
dfcent          = median(ddf1);
% dlcent          = (floor(length(l)/2)+1);
% per Tim - due to filter start-up delay, about 36 samples from the origin
% due to filtering (peak resonpse may not be centered)

% dl              = dl(1:2:end);
DRT1            = DR(ddf1,dl1)/PK;
DRT1dB          = dbp(DRT1);

% PK1             = max(max(DRT1));
% DRT1            = DRT1 / PK1;
[ sidelobe_pwr T1dBsidelobidx] = find(DRT1dB(dfcent,:) > -1);
[ sidelobe_pwr T3dBsidelobidx] = find(DRT1dB(dfcent,:) > -3);
[ sidelobe_pwr T6dBsidelobidx] = find(DRT1dB(dfcent,:) > -6);

[ F1dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -1);
[ F3dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -3);
[ F6dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -6);

% T6dBsidelobidx  = T6dBsidelobidx - 59;      % adjust time peak location
% by windowing and filter delay offset (empirical) - not required after new
% search for indices for computed ambiguity function  - AGM May 2017

DRsidelobes     = DRT1;
DRsidelobes(F6dBsidelobidx,T6dBsidelobidx) = 0;     % zero out mainlobe from calculation
% DRsidelobes(T6dBsidelobidx, F6dBsidelobidx) = 0;
mainlobebins    = length(F6dBsidelobidx) * length(T6dBsidelobidx);

figure(20)
mesh(DRsidelobes)
colorbar
% integrated sidelobes
% exlcude main lobe to -6 dB points, integrate the rest
mainlobePwr     = sum(sum( (DRT1(F6dBsidelobidx,T6dBsidelobidx)/PK).^2 ))/mainlobebins;
mainlobePwrdB   = 10*log10(mainlobePwr);
totalPwr        = sum(sum( (DRT1/PK).^2 ))/mainlobebins;
sidelobePwr     = sum(sum( (DRsidelobes/PK).^2 ))/mainlobebins;
diffPwr         = totalPwr - sidelobePwr;
totalPwrdB      = 10*log10(totalPwr);
sidelobePwrdB   = 10*log10(sidelobePwr);


% clear DRT1 DRT1dB

%% Decimate for display purposes
PK              = max(max(DR));
dfmax           = 5000;
tsmax           = floor(max(l)/OSR*1.0);        % where is the 0.8 factor from ?? (AGM May)
ddf             = find(abs(df)<=dfmax);
ddf             = ddf(1:2:end);
dl              = find(abs(l)/OSR<=tsmax);
dl              = dl(1:2:end);
DRT             = dbp(DR(ddf,dl)/PK);
DRT(DRT<-32)    = -32;                  %% liimt below to focus the display on critical information

%% JanS: PAPR, edits AGM added

pwr = abs(sref).^2;
av_pwr = mean(pwr);
[ pk_pwr pkpidx ] = max(pwr);

papr = 10*log10(pk_pwr/av_pwr);

fprintf('%s, Time Domain Power: avg. %.2f (dB), peak %.2f (dB), peak/avg %.2f (dB) \n', seq_type, 10*log10(av_pwr), 10*log10(pk_pwr), papr);
fprintf('Menu %d, (PS) Ambiguity stats: main lobe %.2f (dB), full ambiguity fn %.2f (dB), peak side %.2f (dB), integrated %.2f (dB) \n', training_type, maindB, totaldB, peakdB, intsidedB);
fprintf('Menu %d, (OS) Ambiguity stats: main lobe %.2f (dB), full ambiguity fn %.2f (dB), sidelobes %.2f (dB), difference %.2f (dB) \n', training_type, mainlobePwrdB, totalPwrdB, sidelobePwrdB, 10*log10(diffPwr) );
fprintf('Main lobe width (zero Doppler offset): -1 dB %.0f (us), -3dB %.0f (us), -6 dB %.0f (us) \n', ...
    length(T1dBsidelobidx)/fs*1e6, length(T3dBsidelobidx)/fs*1e6, length(T6dBsidelobidx)/fs*1e6)
fprintf('Main lobe width (zero Doppler offset): -1 dB %.1f (symb), -3dB %.1f (symb), -6 dB %.1f (symb) \n', ...
    length(T1dBsidelobidx)/OSR, length(T3dBsidelobidx)/OSR, length(T6dBsidelobidx)/OSR)

fprintf('Main lobe width (zero time offset):    -1 dB %.0f (Hz), -3dB %.0f (Hz), -6 dB %.0f (Hz) \n', ...
    length(F1dBsidelobidx)*freqstep, length(F3dBsidelobidx)*freqstep, length(F6dBsidelobidx)*freqstep)
%%

figure(10)
plot((l/OSR),dbp(abs(r)/pk0))
grid
xlabel('Time Offset in Symbols')
ylabel('Auto-Correlation Response (dB)')
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

figure(21)
plot(real(sref), imag(sref))
grid on
title([ 'Real versus imag of training sequence' seq_type ]);

figure(22)
plot(2*dbp(sref))
grid on
title('Time domain reference (training) sequence signal envelope (dB)');
xlabel(['Sample number (OS factor 6)' seq_type ])

figure(23)
sp = abs(fft(sref));
freqa = fs/length(sref)*(0:(length(sref)-1)) - fs/2;
plot(freqa/1e3, fftshift(2*dbp(sp/max(sp))))
grid
title(['Training sequence spectrum' seq_type ])
ylabel('Spectrum (dB)')
xlabel('Frequency (kHz)')

figure(11);                  %% IMAGE Plot
imagesc(l(dl)/(OSR),df(ddf),DRT);
colorbar;
xlabel('Time Delay (Symbols)')
ylabel('Dopppler Offset (Hz)')
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

figure(13);                  %% Quasi-3D 'Mesh'
mesh(l(dl)/OSR,df(ddf),DRT)

colorbar;

xlabel('Time Delay (Symbols)')
ylabel('Dopppler Offset (Hz)')
zlabel('Ambiguity Power')
view(-45,10);
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
axis([ -tsmax tsmax -dfmax dfmax -30 0]);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
%     axis([ -tsmax tsmax -dfmax dfmax -16 0]);
% end

figure(14);                 %% Over plot to summarize peak SL levels
plot(df,dbp(DR(:,dl)/pk0))
grid
xlabel('Carrier Offset in Hz')
ylabel('Doppler Ambiguity Response (dB)')
axis([-dfmax dfmax -30 2])
% axis([-dfmax dfmax -15 20])

% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

