%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    08172016
%%  Free to use for non-commercial, academic or Standards Body use
%%  All rights reserved by author.
%%  Modifications and additions A. Macikunas
%%  Free to use for non-commercial, academic or Standards Body use
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%%  Brute force optimizer for search to find "Best" Peak/Sidelobe ratio
%%  constant envelope Polyphase Traing Sequence.  Optimization is over the
%%  entire Cross-Ambiguity function for a LEO Satellite Receiver in the
%%  international Maritime Mobile band.
%%
%%  Max Doppler + LO Uncertainty for +/-65 deg elevation equates to
%%  ~ +/- 5200 Hz at 162 MHz so this is the minimum search Doppler.
%%  Optimization is with respect to the linear cross-correlation/ambiguity function
%%  which is the relavant method for the orbital receiver due to the operating
%%  mode needed in this CCI limited channel as opposed to Circular Correlation
%%  used to characterize CAZAC sequences, which assumes pre-alerted search.
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% -----------------------------------------------------------------------
%function VDE_TrngSeq_Rev(BARKER)
clear all;
training_type = menu('Select Reference','2092 Barker sequence (from Jan)', 'Dual Barker time reversed AGM', ...
    'Barker (Jan coder)','13-bit Correlator Barker (Jan coder)','Space Norway 27-bit propsed', 'Zadoff Chu 27-symb', ...
    'Zadoff Chu 32-symb', 'Chu 27-symb', 'Chu 32-symb','Polyphase Barker 27 (TD)','Frank 25', ...
    'Unattributed binary 27 (TD)', 'Unfiltered Zadoff-Chu 27', 'AM 69 phase sequence', ...
    '4 FSK like DMR+AGM', '4 FSK like Frank', '4 FSK - future', 'Barker created (TimD rotated QPSK)',  ...
    'Future - various to try');

% training_type = 0;
% close all;

format compact;
format short

cc          = ['b' 'g' 'c' 'm' 'r' 'k'];
BR          = 19200;
OSR         = 10;
OSR         = 20;

fs          = OSR*BR;
NT          = 6;
pi2         = 2*pi;
shapingf    = 0.3;
% shapingf = .25;

%% for FSK modulation
M           = 4;            % Modulation order
k           = log2(M);      % Bits per symbol
SymbR       = BR/k;         % symbol rate (Hz)
% Fs = SymbR * OSR;    % Sample rate (Hz)

nsamp       = fs/SymbR;     % Number of samples per symbol
nsamp       = fs/SymbR/2;     % Number of samples per symbol

%         freqsep = SymbR/8;  % Frequency separation (Hz)
freqsep     = 2 * 1296;  % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X
freqsep     = 4 * 1296;  % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X


if(training_type==1)
    
    % Copy and paste of the VDES seq generated by my code - pi/4-QPSK:
    ref = [0.70711 + 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, 0.70711 + 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, -1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, -1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i];
    seq_type = 'Barker QPSK 2092 Jan';
    
elseif(training_type==2)
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    revbarker = barker(13:-1:1);
    TRNG = [1 barker revbarker];         %% As currently proposed
    %     TRNG = [barker 3 revbarker];         %% As currently proposed
    TRNG = [1 ~barker ~revbarker];         %% As currently proposed
    TRNG = [1 ~barker ~revbarker];         %% As currently proposed -14 dB peak, -3.19 dB integrated
    TRNG = [1 ~barker ~revbarker 0];      %% As currently proposed -13.2, -3.5 dB 28 symbols
    TRNG = [ ~barker 1 revbarker ];      %% As currently proposed -14.1, -3.4 dB 28 symbols
    
    % two more cases to try - stretched barker - repeat each symbol twice
    % interleave forward and reversed Barker sequence, and try with one of
    % them inverted
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    seq_type = 'Barker reversed AGM QPSK';
    
elseif(training_type==3)
    
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker];         %% As currently proposed
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG)) = TRNG;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    seq_type = 'Barker QPSK Jan';
    
elseif(training_type==4)
    
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker];         %% As currently proposed
    TRNGshort = [barker];
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG)) = TRNG;
    trngshort_mapped(1:2:(2*length(TRNGshort)-1)) = TRNGshort;
    trngshort_mapped(2:2:2*length(TRNGshort)) = TRNGshort;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    [refshort] = qpskMapperJan(trngshort_mapped);
    seq_type = '13-bit only Barker QPSK Jan';
    
elseif(training_type==5)
    
    % New Space Norway Training Sequence
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 1 0 1 1 ];  % 1 bit altered - AGM to see impact
    
    TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trng_mapped;
    
    [ref] = qpskMapperJan(trng_mapped);
    seq_type = 'Space Norway 27 symb';
    
elseif(training_type==6)
    
    zad = menu('Which ZadofChu 27-bit sequence','Default', 'Unfiltered');
    
    if(zad==1)
        load ZadofChu_Amb; ref = bb;            %
    else
        load ZadofChu_AmbxxUnFilt; ref = bb;    %% "best" Choice so far: Unfiltered
    end
    seq_type = 'ZadoffChu 27 symb';
    
elseif(training_type==7)
    
    load ZadofChu32_Ambxy; ref = bb;
    seq_type = 'ZadoffChu32-Ambxy';
    
    
elseif(training_type==8)
    load CHU_27; ref = a;  % am - does this exist - need to create it
    seq_type = 'Chu-27';
    
elseif(training_type==9)
    
    load CHU_32; ref = a;  % am - does this exist - need to create it
    seq_type = 'Chu-32';
    
elseif(training_type==10)
    
    load polyphase_barker27; ref = bb;
    seq_type = 'Polyphase Barker Pi/4-QPSK';
    
elseif(training_type==11)
    
    Frank25 = [0 0 0 0 0 0 1 2 3 4 0 2 4 1 3 0 3 1 4 2 0 4 3 2 1];
    ref = exp(1i*pi2*Frank25/5);
    seq_type = 'Frank25';
    
elseif(training_type==12)
    
    [ref, ~] = pi4qpskMapper([0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1], 0);
    
    seq_type = 'Unlabelled binary (TD)';
    
elseif(training_type==13)
    
    load ZadofChu_AmbxxUnFilt; ref = bb;    %% "best" Choice so far: Unfiltered
    
    seq_type = 'ZadoffChu-27 Unfiltered';
    
elseif(training_type==14)
    load AMphase
    
    ref = exp(1i*pi2*phasefn);
    seq_type = 'AM 69 phase sequence';
    
elseif(training_type==15)
    
    data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 ]; % Arunas 4 1st 24-symbols from DMR voice BS bound
    data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 0 2 0 ]; % Arunas 5 1st 24-symbols from DMR voice BS bound + AM
    data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
    data = [ 1 3 0 1 2 1 0 3 2 0 1 2 3 1 0 3 1 3 2 1 0 3 1 3 0 2 0 ]; % Arunas 7 1st 24-symbols from DMR voice BS bound + AM
    data = [[ 1 3 1 1 1 1 3 3 3 1 1 3] ([ 3 1 3 3 1 3 1 1 3 3 1 3 ] - 1) [0 1 2] ]; % Arunas 4 1st 24-symbols from DMR voice BS bound
    data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
    
    txSig = fskmod(data, M,freqsep, nsamp, fs);
    %     load txSigref txSig M
    shapingf = 1;
    
    seq_type = '4FSK DMR+AGM)';
    
elseif(training_type==16)
    
    numbersymb  = 27;
    data = randi([0 M-1],numbersymb,1)';
    data = 2*[ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ]; %space Norway
    data = [ 1 1 1 0 0 0 0 1 1 0 1 1 0 1 1 0 0 1 0 0 1 1 1 1 0 1 0 ]; % Arunas 1
    data = [ 2 1 1 3 0 0 3 1 1 3 1 1 3 1 1 3 0 1 3 0 1 2 1 1 3 1 0 ]; % Arunas 2
    data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 1 4 ] - 1; % Arunas 3 not bad, ISL -4.1 dB
    
    data = randi([0 M-1],numbersymb,1)';
    
    txSig = fskmod(data, M,freqsep, nsamp, fs);
    shapingf = 1;
    
    seq_type = '4FSK Like Frank';
    
elseif(training_type==17)
    
    seq_type = '4FSK future';
    
elseif(training_type==18)
    % JanS: Tim's original code - pi/4 rotated BPSK, this is not pi/4-QPSK!!!!!
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker]          %% As currently proposed
    symbols = [TRNG; TRNG];             %% Set-up for phase map
    TrngSym = [2 1]*symbols;
    TrngSym(TrngSym==3) = 2;
    ref = exp(1i*(TrngSym*pi/2+pi/4));  %% pi/4 rotated BPSK
    %     iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));
    %     sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
    %     sref = sref(ndelay+1:end);
    seq_type = 'Barker pi/r rotated QPSK';
    
elseif(training_type==19)
    
    % Files I don't have from Tim
    %     load Max27P2SL_1; ref = pn0;
    %     load Max32P2SL_TAG3; ref = pn0;
    %     load BestZadoff_ChuRev3; ref = bb;
    %       load BestZadoff_ChuRev3; ref = bb;     %% bb Second-BEST - small search range;
    %     load CHU_33;ref = a;                %% CAZAC Sequence
    %     load tempCHUxy;ref = bb;
    %      load tempChu1; ref = bb;              %% Chu BEST;
    % load tempCHUxx1; ref = bb;            %% very Good Intermediate: Unfiltered
    
end

seq_type = [ seq_type ' shaping ' num2str(shapingf)];

% shaping factor defined above, note a value of  0.3
d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,shapingf);  % rolloff factor is nom 0.3 AGM
% d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,2-1);  % rolloff factor is 0.3 for Beta = 1.3 - 1 AGM

% filter shaping is 2 * NT = 13 taps
% Jan's original
% d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2, 0.3); % Jan: Changed from 1.3 - 1 to 1
% comment says to change to 1, but it is changed back to 0.3 in code provided on Sept 2016

H = design(d);
hsrrc = H.numerator;
hsrrc = hsrrc/sum(hsrrc);
ndelay = (length(hsrrc)-1)/2;
% save ShapingFIR hsrrc ndelay OSR;


%% Basic auto-correlation ----------------------------------------------

%% for PSK modulations
if( training_type < 15  || training_type > 17)
    NN = length(ref);
    
    % reshape and filter reference function
    iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));  %% Over-sampled SRRC'd Modulation format
    sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
    sref = sref(ndelay+1:end);
    % sref = kron(ref,ones(1,OSR));
    % delf = 1500;
    % sref2 = sref;
    % sref = sref.*exp(pi2*1j*delf/fs*(0:(length(sref)-1)));  % implement a frequency shift of reference waveform
    % [r,l] = xcorr(sref,sref2);               %% auto-correlation for reference
else        % for FSK modulations
    %     sref = txSig;
    
    % future modification AGM May 2017 - check for symbol rate scaled SRRC filter corresponds to DMR (shaping factor appears to be 1 for DMR)
    sref = filter(hsrrc,1,[txSig zeros(1,ndelay)]);
    sref = sref(ndelay+1:end);
    NN = length(txSig) / OSR;
end

if(training_type==4)
    irefshort = reshape([refshort.' zeros(length(refshort),OSR-1)].',1,OSR*length(refshort));  %% Over-sampled SRRC'd Modulation format
    srefshort = filter(hsrrc,1,[irefshort zeros(1,ndelay)]);
    srefshort = srefshort(ndelay+1:end);
    
    [r,l] = xcorr(srefshort,sref);               %% auto-correlation for reference
else
    [r,l] = xcorr(sref,sref);               %% auto-correlation for reference
end

[ pk_corr pkcidx ] = max(r);

pk0                 = max(abs(r));

pwr = abs(sref).^2;
av_pwr = mean(pwr);
[ pk_pwr pkpidx ] = max(pwr);

papr = 10*log10(pk_pwr/av_pwr);
fprintf('%s, Time Domain Power: avg. %.2f (dB), peak %.2f (dB). \n\n', seq_type, 10*log10(av_pwr), 10*log10(pk_pwr));
fprintf(' -- Peak/avg %.2f (dB), number of symbols in training sequence. %d \n\n',  papr, NN);

fprintf('ZERO Frequency offset = %.f (Hz), current peak is zero level reference %.2f dB \n', 0, 0);

%%  zero Doppler peak
% single Doppler ambiguity analysis

supressout = 0;
[pks_dB, PKS, sidePKS, peaksubloc, LOCS, W, P, lenpeak, maindB, totaldB, peakdB, intsidedB ] = peakstats( r, l, pk_corr, pk0, seq_type, 0, supressout);

%% Now compute the Cross-Ambiguity over the Satellite Doppler range at coarse intervals
freqstep        = 250;                   % frequency steps - in Hz
df              = -4500:freqstep:4500;
lf              = length(df);
DR              = zeros(lf,length(r));
loc             = 0;
locS            = 0;
peaks           = 0;
sidepeaks       = 0;

fprintf('Frequency offset (Hz), current peak to zero Doppler peak (dB), peak SL at offset, ISLR (dB)\n');

for m=1:lf
    supressout  = 1;
    dref        = sref.*exp(1i*pi*2*df(m)/fs*(0:length(sref)-1));
    [dr,l]      = xcorr(dref,sref);
    dr          = abs(dr);
    %     DR(m,:)     = absdr;                                                     % does not need to be absolute value here
    [pks_dB, PKS, sidePKS, peaksubloc, LOCS, W, P, lenpeak, maindB, totaldB, peakdB, intsidedB ] = peakstats( dr, l, pk_corr, pk0, seq_type, 1, supressout);
    LOCS2       = LOCS;
    if(df(m)<50)
        LOCS2(peaksubloc) = [];       % null out main peak from index of sidelobes
        PKS(peaksubloc)    = [];
    end
    fprintf('      %6.f (Hz),         %5.2f,   %6.2f,   %5.2f (dB) \n', df(m), 20*log10(max(dr)/pk0), 20*log10(max(sidePKS)), intsidedB);
    
    loc         = [loc LOCS2];
    locS        = [locS LOCS2];
    peaks       = [peaks PKS];
    sidepeaks   = [sidepeaks sidePKS];
    
end

intsidedB       = 10*log10( sum(sidepeaks.^2)  );

fprintf('No. of sidelobe peaks found %.0f, %.1f dB integrated level.\n', length(sidepeaks), intsidedB );

sortpks         = sort(sidepeaks,'descend');
figure(15)
bar(20*log10(sortpks))

figure(16)
histogram(20*log10(sidepeaks),31,'BinLimits',[-30,0], 'normalization', 'count')
axis([-30 0 0 100]);
grid;

%% Now compute the Cross-Ambiguity over the Satellite Doppler Extremes
freqstep        = 10;                   % frequency steps - in Hz
df              = -5000:freqstep:5000;
lf              = length(df);
DR              = zeros(lf,length(r));
for m=1:lf
    dref        = sref.*exp(1i*pi*2*df(m)/fs*(0:length(sref)-1));
    [dr,l]      = xcorr(dref,sref);
    DR(m,:)     = abs(dr);                                                          % does not need to be absolute value here
end

%% Window for statistics purposes
PK              = max(max(DR));
dfmax1          = 5000;                 % maximum Doppler plus tolerance, low LEO AGM
% ************ fix - center freq indices not right when I change this to 4250 Hz - can't
% find why?
tsmax1          = floor(max(l)/OSR*0.9);
ddf1            = find(abs(df)<=dfmax1);
% ddf             = ddf(1:2:end);
dl1             = find(abs(l)/OSR<=tsmax1);
% dl1     = l;
dlcent          = floor(length(dl1)/2)+1;
dfcent          = median(ddf1);
% dlcent          = (floor(length(l)/2)+1);
% per Tim - due to filter start-up delay, about 36 samples from the origin
% due to filtering (peak resonpse may not be centered)

% dl              = dl(1:2:end);
DRT1            = DR(ddf1,dl1)/PK;
DRT1dB          = dbp(DRT1);

% PK1             = max(max(DRT1));
% DRT1            = DRT1 / PK1;
[ sidelobe_pwr T1dBsidelobidx] = find(DRT1dB(dfcent,:) > -1);
[ sidelobe_pwr T3dBsidelobidx] = find(DRT1dB(dfcent,:) > -3);
[ sidelobe_pwr T6dBsidelobidx] = find(DRT1dB(dfcent,:) > -6);

[ F1dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -1);
[ F3dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -3);
[ F6dBsidelobidx sidelobe_pwr ] = find(DRT1dB(:,dlcent) > -6);

% T6dBsidelobidx  = T6dBsidelobidx - 59;      % adjust time peak location
% by windowing and filter delay offset (empirical) - not required after new
% search for indices for computed ambiguity function  - AGM May 2017

DRsidelobes     = DRT1;
DRsidelobes(F6dBsidelobidx,T6dBsidelobidx) = 0;     % zero out mainlobe from calculation
% DRsidelobes(T6dBsidelobidx, F6dBsidelobidx) = 0;
mainlobebins    = length(F6dBsidelobidx) * length(T6dBsidelobidx);

figure(20)
mesh(DRsidelobes)
colorbar
% integrated sidelobes
% exlcude main lobe to -6 dB points, integrate the rest
% mainlobePwr     = sum(sum( (DRT1(F6dBsidelobidx,T6dBsidelobidx)/PK).^2 ))/mainlobebins;
% mainlobePwrdB   = 10*log10(mainlobePwr);
% totalPwr        = sum(sum( (DRT1/PK).^2 ))/mainlobebins;
% sidelobePwr     = sum(sum( (DRsidelobes/PK).^2 ))/mainlobebins;
% diffPwr         = totalPwr - sidelobePwr;
% totalPwrdB      = 10*log10(totalPwr);
% sidelobePwrdB   = 10*log10(sidelobePwr);


% clear DRT1 DRT1dB

%% Decimate for display purposes
PK              = max(max(DR));
dfmax           = 5000;
tsmax           = floor(max(l)/OSR*1.0);        % where is the 0.8 factor from ?? (AGM May)
ddf             = find(abs(df)<=dfmax);
ddf             = ddf(1:2:end);
dl              = find(abs(l)/OSR<=tsmax);
dl              = dl(1:2:end);
DRT             = dbp(DR(ddf,dl)/PK);
DRT(DRT<-32)    = -32;                  %% liimt below to focus the display on critical information

%% JanS: PAPR, edits AGM added

pwr = abs(sref).^2;
av_pwr = mean(pwr);
[ pk_pwr pkpidx ] = max(pwr);

papr = 10*log10(pk_pwr/av_pwr);

% fprintf('Menu %d, (OS) Ambiguity stats: main lobe %.2f (dB), full ambiguity fn %.2f (dB), sidelobes %.2f (dB), difference %.2f (dB) \n', ...
%     training_type, mainlobePwrdB, totalPwrdB, sidelobePwrdB, 10*log10(diffPwr) );
fprintf('\nMain lobe width (zero Doppler offset): -1 dB %.0f (us),    -3dB %.0f (us),    -6 dB %.0f (us) \n', ...
    length(T1dBsidelobidx)/fs*1e6, length(T3dBsidelobidx)/fs*1e6, length(T6dBsidelobidx)/fs*1e6)
fprintf('Main lobe width (zero Doppler offset): -1 dB %.1f (symb), -3dB %.1f (symb), -6 dB %.1f (symb) \n', ...
    length(T1dBsidelobidx)/OSR, length(T3dBsidelobidx)/OSR, length(T6dBsidelobidx)/OSR)

fprintf('Main lobe width (zero time offset):    -1 dB %.0f (Hz),   -3dB %.0f (Hz),   -6 dB %.0f (Hz) \n', ...
    length(F1dBsidelobidx)*freqstep, length(F3dBsidelobidx)*freqstep, length(F6dBsidelobidx)*freqstep)
%%

figure(10)
plot((l/OSR),dbp(abs(r)/pk0))
grid
xlabel('Time Offset in Symbols')
ylabel('Auto-Correlation Response (dB)')
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
axis([-30 30 -60 0]);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

figure(21)
plot(real(sref), imag(sref))
grid on
title([ 'Real versus imag of training sequence' seq_type ]);

figure(22)
plot(2*dbp(sref))
grid on
title('Time domain reference (training) sequence signal envelope (dB)');
xlabel(['Sample number (OS factor 6)' seq_type ])

figure(23)
sp = abs(fft(sref));
freqa = fs/length(sref)*(0:(length(sref)-1)) - fs/2;
plot(freqa/1e3, fftshift(2*dbp(sp/max(sp))))
grid on
axis([-50 50 -80 10])
hold on
plot(-10:10,0*ones(21,1),':k')
plot([10 25],[-25 -70],':k')
plot([10 10],[-25 0],':k')
plot([-10 -10],[-25 0],':k')
plot([-25 -10],[-70 -25],':k')
plot([-62.5 -25], [-70 -70], ':k')
plot([62.5 25], [-70 -70], ':k')
title(['Training sequence spectrum' seq_type ])
ylabel('Spectrum (dB)')
xlabel('Frequency (kHz)')
hold off

figure(11);                  %% IMAGE Plot
imagesc(l(dl)/(OSR),df(ddf),DRT);
colorbar;
xlabel('Time Delay (Symbols)')
ylabel('Dopppler Offset (Hz)')
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

figure(13);                  %% Quasi-3D 'Mesh'
mesh(l(dl)/OSR,df(ddf),DRT)

colorbar;

xlabel('Time Delay (Symbols)')
ylabel('Dopppler Offset (Hz)')
zlabel('Ambiguity Power')
view(-45,10);
% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
axis([ -tsmax tsmax -dfmax dfmax -30 0]);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
%     axis([ -tsmax tsmax -dfmax dfmax -16 0]);
% end

figure(14);                 %% Over plot to summarize peak SL levels
plot(df,dbp(DR(:,dl)/pk0))
grid
xlabel('Carrier Offset in Hz')
ylabel('Doppler Ambiguity Response (dB)')
axis([-dfmax dfmax -30 2])
% axis([-dfmax dfmax -15 20])

% if(~training_type)
title([num2str(NN),' symbol ' seq_type ' ']);
% else
%     title('Currently proposed Barker-based VDE Training Sequence');
% end

function [pks_dB, PKS, sidePKS, peaksubloc, LOCS, W, P, lenpeak2, maindB, totaldB, peakdB, intsidedB ] = peakstats( r, l, pk_corr, pk0, seq_type, holdgraph, supressout);

%% single Doppler ambiguity analysis
% rscale              = abs(r) / pk_corr;
rscale              = abs(r) / pk0;

[PKS,LOCS,W,P]      = findpeaks( rscale, l);

figure(29)
if(holdgraph)
    hold on
else
    hold off
end
findpeaks( rscale, l, 'Annotate', 'extents')
%%eliminate non-prominent peaks (not obviously a peak, empirical threshold -35 dB), and those peaks
% below -30 dB
pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);
lenpeak0            = length(PKS);

[a1, b1]            = find(pks_dB < -30);
PKS(b1)             = [];
LOCS(b1)            = [];
W(b1)               = [];
P(b1)               = [];
lenpeak1            = length(PKS);

pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);

[a2, b2]            = find(prom_pks_dB < -35);
PKS(b2)             = [];
LOCS(b2)            = [];
W(b2)               = [];
P(b2)               = [];
lenpeak2            = length(PKS);

[a3, peaksubloc]    = max(PKS);
pks_dB              = 20*log10( PKS );
prom_pks_dB         = 20*log10(P./PKS);

% alternate sidelobe measurements based on identified peaks of single
% Doppler cross-correlation function
maindB              = pks_dB(peaksubloc);
sidePKS             = PKS;
sidePKS(peaksubloc)     = [];
% maindB      = max(pks_dB)
sidePKSdB           = 20*log10( sidePKS);

totaldB             = 10*log10(sum(PKS.^2));
peakdB              = 20*log10(max(sidePKS));
% intsidedB     = 10*log10( sum(PKS.^2) - PKS(peaksubloc).^2 )
intsidedB           = 10*log10( sum(sidePKS.^2)  );

% diffPwr         = totalPwr - sidelobePwr;
% totalPwrdB      = 10*log10(totalPwr);
% sidelobePwrdB   = 10*log10(sidelobePwr);

figure(28)
if(holdgraph)
    hold on
else
    hold off
    grid
end

histogram(sidePKSdB,7,'BinLimits',[-35 0])
axis([-35 0 0 25])
title('Histogram of distribution of peaks in different attentuation ranges (by number of peaks)')

figure(12)
if(holdgraph)
    hold on
    grid on
else
    hold off
    grid on
end

plot(l,20*log10(abs(r/pk0)), '.');
hold on;
plot(LOCS, pks_dB, '+r');
%plot(LOCS, prom_pks_dB, '*g');
plot(LOCS, 20*log10(P), '*g');
title(['Peaks of cross-correlation function, ' seq_type]);
xlabel('lag offset')
ylabel('Response (dB)');
axis([-600 600 -30 2]);
hold off;
% pause(0.2)

if(supressout==0)
    fprintf('XCorr stats: peak %.2f (dB), all lrg peaks %.2f (dB), peak side %.2f (dB), ISLR %.2f (dB) \n',  maindB, totaldB, peakdB, intsidedB);
    fprintf('No. of peaks analyzed %.0f, %.0f below -30 dB(Pk) , %.f non-prominent peaks.\n', lenpeak2,  lenpeak0 - lenpeak1, lenpeak1-lenpeak2);
end
end
