%% Working Group 5 - Training Sequence Analysis
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    07112016
%%  Additions and edits A. Macikunas, 30052017, 01062017, 04062017

%% updates Aug 2017 to fix random data after reference waveform (previous was incorrect QPSK waveform) file: ... AM5R3 this version
%% updates Aug 2017 to add option for coloured noise (starting ... AM6R3)
%% updates Aug 2017 for Krzysztof and Space Norway sequence 'detectors' (starting ... AM7R3)

%%  Free to use for non-commercial, academic or Standards Body Purposes
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Implements a poor-mans ROC curve estimation for a specific Tarining sequence
%% in a QPSK Modulated message with a BPOSK or other form of training sequence
%% Additions to analysis, new waveforms, updated and new plots, SNR loop
%%   diagnostic data collection for examination of detector statistics AGM May 2017
%% Outputs:
%%     Plots ROC Curves for study, changed to fixed threshold ROC, remove CFAR estimation/threshold
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
clear all;
r2      = sqrt(2);

Ntrials             = 100000;           %% Need to calcualte this for Statistical Significance
% Ntrials = 100, 30 seconds approx, Ntrials = 10000, 15 minutes, 100000 = hour +?
Ntrials             = 10000;           %% Need to calcualte this for Statistical Significance 15 min
Ntrials             = 1000;           %% Need to calcualte this for Statistical Significance 1 min?
Ntrials             = 200;              %% 30 seconds?
Ntrials             = 500;              %% 30 seconds?
Ntrials             = 100;              %% 30 seconds?

cc                  = ['b' 'g' 'c' 'm' 'r' 'k'];
BR                  = 19200;     % QPSK bit rate
% OSR         = 20;
OSR                 = 8;

fs                  = OSR*BR;
NT                  = 6;                % shaping filter parameter
pi2                 = 2*pi;
shapingf            = 0.3;              % default SRRC shaping factor
% shapingf = .25;

%% for FSK modulation
M                   = 4;                % Modulation order for FSK ( = 4 FSK )
k                   = log2(M);          % Bits per symbol
SymbR               = BR/k;             % symbol rate (Hz)
% Fs = SymbR * OSR;         % Sample rate (Hz)

% nsamp       = fs/SymbR;       % Number of samples per symbol
nsamp               = fs/SymbR/2;       % Number of samples per symbol

%         freqsep = SymbR/8;  % Frequency separation (Hz)
% freqsep     = 2 * 1296;       % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X
freqsep             = 4 * 1296;         % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X
% freqsep     = 3.3 * 1296;  % Frequency separation (Hz) 1296 Hz is separation for ETSI DRM standard, use 4X

outerloop           = menu('Which type of loop', 'SNR loop', 'Doppler loop', 'Doppler loop narrow');

training_type       = [];
trcases = [ 2 3 4 5 6 8 10 12 13 15 16 17];
trcases = [ 2 4 ];
trcases = [ 1 2 ];
trcases = [ 16 ];
trcases = [ 17 ];
trcases = [ 3 ];

% trcases = [ 2 3 4 5 12 ];

% trcases = [7 9 11 14 ];

for kk = 1:length(trcases)
    
    training_type = trcases(kk);
    
    [shapingf_override, training_type, seq_type, data, ref, refshort, reflen, txSig, txSiglen] = ...
        select_sequenceR3( M,freqsep, nsamp, fs, training_type);
    
    if( ~isempty(shapingf_override) )
        shapingf        = shapingf_override;
    end
    
    seq_type = [ seq_type ', shaping = ' num2str(shapingf)];
    
    %% ---------------------------------------------------
    % body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));          %  random pi/4 QPSK? AGM
    % body(2:2:end)   = body(2:2:end)*exp(1i*pi/4);                           % this is not pi/4 QPSK - deleted, unnecessary
    
    % OSR                 = 8;
    % NT                  = 6;                %% Set-up the SRRC Shaping filter
    alpha               = shapingf;         %% BW Excess factor
    d                   = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,alpha);
    H                   = design(d);
    hsrrc               = H.numerator;
    hsrrc               = hsrrc/sum(hsrrc);
    ndelay              = (length(hsrrc)-1)/2;
    
    % add FSK pre-modulated signal here?  AGM June 2017
    if( training_type < 21  || training_type > 24)      % PSK sequences
        %     if( training_type < 15  || training_type > 17)
        
%         body            = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));          %  random pi/4 QPSK? AGM - ans = no, revise
        
        body_symb       = floor(rand(1,2*1245)*2);          %  random bits

        [body, ~]       = pi4qpskMapper(body_symb,0);
        
        inBB            = [ref body];     %% Test Message at BB
        
        xin             = kron(inBB,ones(1,OSR));
        lTrng           = length(ref);
        ltrain          = lTrng*OSR;
        
    else        % 4FSK sequence
        % hard code sequence from DMR+AGM customization
        %     data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
        % variable data should be supplied by routine select_sequence
        lTrng           = length(data);
        body            = floor(rand(1,1245)*4);          %  random 4FSK symbols AGM
        data            = [data body];
        
        xin             = fskmod(data, M,freqsep, nsamp, fs);
        
        ltrain          = lTrng*OSR;
        %     xin             = inBB;     % already resampled - no need to change
        %     ltrain          = length(ref);
        
    end
    
    sig                 = filter(hsrrc,1,[xin zeros(1,ndelay)]);
    
    sig                 = sig(ndelay+1:end);    %% Shaped, modulated Test Waveform
    lsig                = length(sig);
    ref                 = sig(1:ltrain);        %% Extract the ideal Training F for later detection processing
    
    SNR                 = -6:1:10;               %% Set an SNR (Es/No) range for ROC Curve
    SNR                 = -9:3:3;               %% Set an SNR (Es/No) range for ROC Curve
    % SNR                 = -9:3:0;               %% Set an SNR (Es/No) range for ROC Curve
    if(outerloop==1)
        Nsnr                = length(SNR);
    elseif(outerloop==2)
        Doppler         = -4:1:4;
        Nsnr            = length(Doppler);
    elseif(outerloop==3)
        Doppler         = -.6:.1:.6;
        Doppler         = 0;
        Doppler         = -.5:.1:.5;
        
        Nsnr            = length(Doppler);
    end
    
    
    %% Typically, 90% Confidence intervals
    k                   = 1;
    Kt                  = 2.9417;                %% Detection threshold: based on Linear Envelope Detector (see Whalen's Text
    %% Or use Robertson's curves for Deg of Freedom
    KD                  = 0;
    initialsize         = 21;
    det                 = zeros(1,initialsize);
    far                 = det;
    % snr                 = 10^(-3/20);
    snr                 = 10^(-3/20);       % default SNR for Doppler series
%     snr                 = 10^(-6/20);

    % snr = 10^(0/20);
    
    Ktmat               = 2:(1/3):7.25;
    
    %% Create reference level and peak location - noiseless
    x                   = [zeros(1,lsig) sig zeros(1,lsig)];     %% create message WF without noise for pk and std dev estimate
    [r,l]               = xcorr(ref,x);       %% Cross Correlation Detection of Rx signal with itself
    rstdRef             = std(r);
    [rmaxRef, mxidxRef] = max(abs(r));
    clear x r l
    CFAR                = 0;        % if CFAR = 1, use CFAR method, else fixed threshold
    
    if( training_type < 16  || training_type > 16)    
        rmax_meanidx        = 40704;
    else
        rmax_meanidx        = 40704+160;  % offset for AIS
    end
    
    % for Kt = 2:(1/3):7.25       %% Vary the detection threshold to sweep out PD v. PFA curve
    for m = 1:Nsnr       %% Vary the detection threshold to sweep out PD v. PFA curve
        clear jthresh
        if(outerloop==1)
            snr             = 10^(SNR(m)/20);
            doppler         = 0;
        else
            snr             = snr;
            doppler         = Doppler(m);
        end
        det(k) = 0;
        far(k) = 0;
        % m is outer loop - over SNR, threshold (Tim D original), Doppler (AGM)
        for j=1:Ntrials
            %         noise = cawgn(3*lsig)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No
            noise       = wgn(1,3*lsig,0)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No Modifed AGM used Matlab White Gaussian Noise, normalized power, lenght = arg
            %         sref.*exp(1i*pi*2*doppler*1000/fs*(0:length(sig)-1))
            %         x           = noise + [zeros(1,lsig) sig zeros(1,lsig)];     %% place message WF in AWGN
            x           = noise + [zeros(1,lsig) sig.*exp(1i*pi*2*doppler*1000/fs*(0:length(sig)-1)) zeros(1,lsig)];     %% place message WF in AWGN
            
            [r,l]       = xcorr(x,ref);       %% Cross Correlation Detection of Trng Seq
            
            [mx, mxidx] = max(abs(r));
            rmax(m,j)   = mx;
            rmaxidx(m,j)= mxidx - rmax_meanidx;            % empirical center index
            %         rmaxidx(m,j) = mxidx - mxidxRef;
            
            y = abs(r(l>-1));       %% Trng @ y(10177)  %% only peak and positive lags are assessed - due to symmetry to save time?  AGM
            if(CFAR==1)
                %% Apply CFAR Detection procedure:  use Slotted Window noise mean estimator
                norm        = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
                y           = y(121:end-50);          %% remove delay and align data and noise level estimate
                norm        = norm(121:end-50);
                % plot((1:length(y))/OSR,y-Kt*norm);pause(0.5);
                % hold on;
                [pk,kp]     = max(y-Kt*norm);       %% Select the peak threshold exceedence
                if(pk>KD)                       %% Use only if y(kp) > KD*norm(kp)
                    if(abs(kp-10056)<3)
                        det(k) = det(k) + 1;
                    end
                end
            end
            %% FAR detection
            [w,l] = xcorr(noise,ref);       %% Cross Correlation Detection of Trng Seq (Noise only)
            
            [mx, mxidx] = max(abs(w));
            nmax(m,j)   = mx;
            nmaxidx(m,j)= mxidx - rmax_meanidx;            % empirical center index
            if(CFAR==1)
                y = abs(w(l>-1));
                norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
                y = y(121:end-50);
                norm = norm(121:end-50);
                % plot((1:length(y))/OSR,y-Kt*norm,'g--');pause(0.5);
                % hold off;
                [pk,kp] = max(y-Kt*norm);
                if(pk>KD)
                    far(k) = far(k) + 1;
                end
            end
            
        end
        k = k+1;
    end
    % kmax = k - 1;
    kmax            = Nsnr;
    pk_max          = max([ max(max(rmax)) max(max(nmax))]);
    pk_min          = min([ min(min(rmax)) min(min(nmax))]);
    % highSNRmidpoint = max(nmax(kmax,:)) - min(rmax(kmax,:));
    highSNRmidpoint = ( mean(rmax(kmax,:)) + mean(nmax(kmax,:)) ) /2;
    % highSNRmidpoint = ( mean(rmax(kmax,:))
    
    numsteps        = 81;
    % numsteps        = 21;
    numsteps        = 121;
    
    numcenterpoints = fix( numsteps/1.5 );
    numtopbottom    = fix( (numsteps - numcenterpoints) / 2);
    topbottomscale  = 0.6;      % empirically chosen to stretch the middle section of threshold
    topbottomscale  = 0.4;      % empirically chosen to stretch the middle section of threshold
    
    jthresh(1:numtopbottom) = (pk_min + (1:numtopbottom)*(highSNRmidpoint-pk_min)*topbottomscale/(numtopbottom)) ;
    jthresh((numtopbottom+1):(numsteps-numtopbottom)) = (1:numcenterpoints+1)*(highSNRmidpoint-jthresh(numtopbottom))/(numtopbottom) + jthresh(numtopbottom);
    % jthresh( (numsteps-numtopbottom+1):numsteps) = (highSNRmidpoint+pk_min + (1:numtopbottom)*(highSNRmidpoint+pk_min)/(numtopbottom)) ;
    jthreshmidmax = jthresh(end);
    jthreshmidlen = length(jthresh);
    nextidx = (numsteps-numtopbottom+1);
    % jthresh( (numsteps-numtopbottom+1):numsteps) = (pk_max - (numtopbottom-1:-1:0)*(pk_max-highSNRmidpoint)*topbottomscale/(numtopbottom)) ;
    % jthresh( (numsteps-numtopbottom+1):numsteps) = (pk_max - (numtopbottom-1:-1:0)*( pk_max- jthreshmidmax )*topbottomscale/(numtopbottom)) ;
    jthresh( (numsteps-numtopbottom+1):numsteps) = (pk_max - ((numtopbottom-1):-1:0)*( pk_max- jthreshmidmax )/(numtopbottom)) ;
    
    [tmp,midjthreshidx]     = min(abs(jthresh-highSNRmidpoint));        % middle threshold, signal to noise high SNR
    
    %  loop to run detection statistics and false alarms as a function of
    %  threshold, also to consider detections based on correct peak location
    if(CFAR==0)
        det                 = zeros(kmax,numsteps);
        far                 = det;
        detpeak             = det;
        detpeak3            = det;
        detpeak5            = det;
        detpeak80           = det;
        rmsSymError         = det;
        % kmax is out loop, SNR, Doppler, etc.
        % numsteps is the number of divisions in the detetion criteria (threshold)
        for m = 1:kmax
            for j = 1:numsteps
                %             jthresh = pk_min + (j-1)*(pk_max-pk_min)/(numsteps);
                %             if(m==1)
                %             threshmat(j) = jthresh;
                %             end
                [a, b]      =  find( rmax(m,:) > jthresh(j));
                
                det(m,j)    = det(m,j) + length( a );
                
                % detections at locations in b
                
                [a, b1]      = find( abs(rmaxidx(m,b)) < 1);
                detpeak(m,j)   = detpeak(m,j) + length( a );
                
                [a, b1]      = find( abs(rmaxidx(m,b)) < 2);
                detpeak3(m,j)   = detpeak3(m,j) + length( a );
                
                [a, b1]      = find( abs(rmaxidx(m,b)) < 3);
                detpeak5(m,j)        = detpeak5(m,j) + length( a );
                
                [a, b1]      = find( abs(rmaxidx(m,b)) < 41);
                detpeak80(m,j)        = detpeak80(m,j) + length( a );
                % on calculate rms error when there are valid peaks within 40
                % samples of target location, and at least 3 points
                if(length(b1)>5)
                    rmsSymError(m,j)      = std(rmaxidx(m,b1));
                else
                    rmsSymError(m,j)      = NaN;
                end
                [a, b] =   find( nmax(m,:)>jthresh(j));
                far(m,j) = far(m,j) + length( a );
            end
        end
    end
    rmsSymError = rmsSymError / OSR;
    
    threshmat       = jthresh;
    fprintf('\n %s \n\n', seq_type)
    threshPD        = [0.25 0.50 0.90]*100;
    threshPFA       = [0.01 0.10 0.50]*100;
    for k = 1:kmax
        % at the PD 25, 50 and 90% detection points, and corresponding FA rates
        % alternately find PD at 1, 10 and 50% false alarm rates
        %
        for l = 1:length(threshPD)
            [a, b] = min( abs( ( detpeak3(k,:) /Ntrials*100 ) - threshPD(l) ) );
            %         PDD(k,l) = a;
            PDD(k,l) =  detpeak3(k,b) /Ntrials*100;
            PDFA(k,l) = far(k,b)/Ntrials*100;
            
            [a, b] = min( abs( ( far(k,:) /Ntrials*100 ) - threshPFA(l) ) );
            PFAD(k,l) = detpeak3(k,b)/Ntrials*100;
            %         PFAFA(k,l) = a;
            PFAFA(k,l) = far(k,l) /Ntrials*100;
        end
        if(outerloop==1)
            fprintf('SNR %3.0f (dB), mid S+N-N threshold, (+/- 1 sample) PD %.2f, PFA %.2f (%%) \n', ...
                SNR(k), detpeak3(k,midjthreshidx)/Ntrials*100, far(k,midjthreshidx)/Ntrials*100);
        else
            fprintf('Doppler %5.0f (Hz), mid S+N-N threshold, (+/- 1 sample) PD %.2f, PFA %.2f (%%) \n', ...
                Doppler(k)*1000, detpeak3(k,midjthreshidx)/Ntrials*100, far(k,midjthreshidx)/Ntrials*100);
            fprintf('  PD driven PFA (PD=25,50,90), PFA = %4.1f, %4.1f, %4.1f (%%), actual Pd = %4.1f, %4.1f, %4.1f (%%) \n', ...
                PDFA(k,:), PDD(k,:));
            fprintf('  PFA driven PD (PFA=1,10,50), PD  = %4.1f, %4.1f, %4.1f (%%), actual PFA= %4.1f, %4.1f, %4.1f (%%) \n', ...
                PFAD(k,:),PFAFA(k,:));
        end
        fprintf('   ----> symbol peak standard deviation %.2f (Symb), median peak location (actual) %.0f (samples)\n\n', ...
            rmsSymError(k,midjthreshidx), rmax_meanidx + median(rmaxidx(k,:)));
    end
    fprintf('All loops, %.0f points, mid S+N-N threshold, (+/- 1 sample) PD %.2f, PFA %.2f (%%) \n', ...
        prod(size(rmsSymError)), sum(detpeak3(:,midjthreshidx))/Ntrials/kmax*100, sum(far(:,midjthreshidx))/Ntrials/kmax*100);
    fprintf('   ----> peak correlated standard deviation %.2f (Symb), median peak location (actual) %.0f (samples)\n', ...
        std(rmsSymError(:,midjthreshidx)), rmax_meanidx + median(median(rmaxidx)));
        fprintf('   ----------------------------------------------------------------------------- \n\n')
    % median(rmaxidx,2)', std(rmaxidx')/OSR       % median value, rms error in estimate in symbols
    % rmsSymError;
    
    % May 28, 2017 - error message when running space Norway case, SNR is 7 long, det is 16
    % long, error because the number of detections is over the threshold values
    % Kt, which in Tim's code sweeps out 16 values
    % Error using plot
    % Vectors must be the same length.
    % Error in TrainingDetectionAM (line 137)
    % plot(SNR,det/Ntrials,'o-');grid on;
    % plot(SNR,det/Ntrials,'o-');grid on;
    
    ax = gca;
    set(0, 'DefaultAxesFontWeight', 'normal', ...
        'DefaultAxesFontSize', 12, ...
        'DefaultAxesFontAngle', 'normal', ... % Not sure the difference here
        'DefaultAxesFontWeight', 'normal', ... % Not sure the difference here
        'DefaultAxesTitleFontWeight', 'normal', ...
        'DefaultAxesTitleFontSizeMultiplier', 1) ;
    
    figure(11)
    % plot(Ktmat,det/Ntrials,'o-b');grid on;        % Tim Dyson's version
    plot(threshmat,det/Ntrials,'o-b');grid on;
    hold on
    % plot(Ktmat,far/Ntrials,'o-r');
    plot(threshmat,far/Ntrials,'o-r');
    hold off
    xlabel('Detection threshold')
    ylabel('P_D')
    title('Detection rate, false alarm rate versus threshold')
    legend('Blue - detections', 'Red - false alarms');
    
    figure(12)
%     loglog(far/Ntrials,det/Ntrials)
    loglog(far'/Ntrials,det'/Ntrials)

    ylabel('P_D')
    xlabel('P_F_A')
    grid
    title('Peak Exceedance vs. false alarm rate log')
    
    figure(32)
%     loglog(far/Ntrials,det/Ntrials)
    loglog(far'/Ntrials,detpeak(:,:)'/Ntrials)

    ylabel('P_D')
    xlabel('P_F_A')
    grid
    title('Detection vs. false alarm rate log')
    % legend to fix
    
%     for j = 1:11; txt([j]) = ['a ' num2str(SNR(j))]; end;
    txt = ['a ' num2str(SNR(1))];
    loglog(far'/Ntrials,detpeak(:,:)'/Ntrials, 'DisplayName', txt );
    legend('show')

    
    %close(35)
    figure(1);
    subplot(111)
    plot( jthresh, detpeak3(:,:)' /Ntrials*100,'.');hold on; plot( jthresh, far(:,:)' /Ntrials*100,'.'); hold off; grid
    title(['Detection and false alarm rate log '  seq_type])
    ylabel('PD and PFA')
    xlabel('Threshold value');
    
    figure(2);
    subplot(111)
    plot( far(:,:)' /Ntrials*100, detpeak(:,:)' /Ntrials*100,'.'); grid on 
%     plot( far(:,:) /Ntrials*100, detpeak(:,:) /Ntrials*100,'.'); grid on

    title(['Detection vs. false alarm rate log ' seq_type])
    ylabel('PD and PFA')
    xlabel('PFA');
    
    figure(13)
    plot(far'/Ntrials,det'/Ntrials,':+')
    ylabel('P_D')
    xlabel('P_F_A')
    grid
    title('All Detections vs false alarm rate')
    
    figure(33)
    plot(far'/Ntrials,detpeak5'/Ntrials,':+')
    ylabel('P_D')
    xlabel('P_F_A')
    grid
    title('Low Timing Error Detection vs false alarm rate - linear')
    
    figure(34)
    plot(far'/Ntrials,detpeak3'/Ntrials,':+')
    ylabel('P_D')
    xlabel('P_F_A')
    grid
    title('Very Low Timing Error Detection vs false alarm rate - linear')
    
    figure(14)
    if(outerloop==1)
        plot(SNR, rmax,'.k')
        hold on
        plot(SNR, nmax,'.r')
        xlabel('SNR (dB)')
        % plot(1:k,rmaxRef,'g')
    else
        plot(Doppler, rmax,'.k')
        hold on
        plot(Doppler, nmax,'.r')
        xlabel('Doppler offset (kHz)')
    end
    hold off
    ylabel('value')
    grid
    title(['Rmax and nmax trend, seq type ' seq_type])
    
    figure(15); plot(rmaxidx,'-')
    title('Index location of Rmax peak value (note:  zoom in on dense area)')
    grid
    
    figure(16);
    histogram(rmaxidx(end,:))
    title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
    grid
    
    figure(17);
    histogram(rmaxidx - mean(rmaxidx), 60)
    % title(['Histogram of Index of peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
    title(['Histogram of Index of peak values, all SNRs = ' seq_type ])
    ylabel('Frequency of Occurance')
    xlabel(['Offset from nominal center (samples), ' num2str(OSR) ' samples per symbol']);
    grid
    
    figure(18);
    histogram(rmaxidx - mean(rmaxidx), 21, 'BinEdges', -10:10, 'Normalization', 'probability' )
    title(['Histogram of peak location, all SNR, zoomed in, ' seq_type ])
    ylabel('Probability')
    xlabel(['Offset from nominal center (samples), ' num2str(OSR) ' samples per symbol']);
    grid
    
    figure(19)
    histogram(rmax)
    hold on
    histogram(nmax)
    hold off
    grid
    title(['Histogram of Rmax and Nmax peak values, SNR = ' num2str(20*log10(snr)) ' dB' ])
    xlabel(['Detection peak sample number, sequence:  ' seq_type]);
    legend('Signal','Noise')
    
    figure(30); plot(rmax, rmaxidx,'.:')
    title(' rmax peak versus rmax peak index (error)')
    
    if(Ntrials>99000)
        %  save ROC_BestModZadof_Chu_CurveTrngDet_p0dB det far Ntrials SNR;
        %  save ROC_SpaceNorway_CurveTrngDet_p0dB det far Ntrials SNR;
        save ROC_BarkerStd_CurveTrngDet_p0dB det far Ntrials SNR;
    end
    
end % automation loop
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%