%% Working Group 5 - Training Sequence Analysis
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    07112016
%%  Free to use for non-commercial, academic or Standards Body Purposes
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%% Implements a poor-mans ROC curve estimation for a specific Tarining sequence 
%% in a QPSK Modulated message with a BPOSK or other form of training sequence
%%
%% Outputs:
%%     Plots ROC Curves for study
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ---------------------------------------------------------------------
clear all;
r2 = sqrt(2);

BARKER = 1;
% load VDE_TrngBarker;            %% sref @ OSR = 8;
if(BARKER)
    %% Define the proposed Barker-based Training Sequence
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker];
    symbols = [TRNG; TRNG];
    TrngSym = [2 1]*symbols;
    k3 = find(TrngSym==3);
    TrngSym(k3) = 2;
    lTrng = length(TrngSym);
    TRNG = exp(1i*(TrngSym*pi/2+pi/4));
else
  load ZadofChu_AmbxxUnFilt; TRNG = bb;  %% "best" 27 symbol Choice so far: Unfiltered 
  lTrng = length(TRNG);
end
%% ---------------------------------------------------
body = exp(1i*(pi/4 + pi/2*floor(rand(1,1245)*4)));
body(2:2:end)= body(2:2:end)*exp(1i*pi/4); 

OSR = 8;
NT = 6;         %% Set-up the SRRC Shaping filter
alpha = 0.3;    %% BW Excess factor
d  = fdesign.pulseshaping(OSR,'Square Root Raised Cosine','Nsym,Beta',NT*2,alpha);
H = design(d);
hsrrc = H.numerator;
hsrrc = hsrrc/sum(hsrrc);
ndelay = (length(hsrrc)-1)/2;

inBB = [TRNG body];     %% Test Message at BB
xin = kron(inBB,ones(1,OSR));
sig = filter(hsrrc,1,[xin zeros(1,ndelay)]);

sig = sig(ndelay+1:end);    %% Shaped, modulated Test Waveform
lsig = length(sig);
ltrain = lTrng*OSR;
ref = sig(1:ltrain);        %% Extract the ideal Training F for later detection processing

SNR = -1:1:5;               %% Set an SNR (Es/No) range for ROC Curve
Nsnr = length(SNR);
Ntrials = 100;           %% Need to calcualte this for Statistical Significance
                            %% Typically, 90% Confidence intervals
k = 1;
Kt = 2.9417;                %% Detection threshold: based on Linear Envelope Detector (see Whalen's Text
                            %% Or use Robertson's curves for Deg of Freedom
KD = 0;
det = zeros(1,10);
far = det;
snr = 10^(SNR(2)/20);
for Kt = 2:(1/3):7.25       %% Vary the detection threshold to sweep out PD v. PFA curve
    det(k) = 0;
    far(k) = 0;
    for j=1:Ntrials
%         noise = cawgn(3*lsig)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No
        noise = wgn(1,3*lsig,0)*std(sig)*sqrt(OSR)/snr/r2;   %% Set the Es/No Modifed AGM used Matlab White Gaussian Noise, normalized power, lenght = arg

        x = noise + [zeros(1,lsig) sig zeros(1,lsig)];     %% place message WF in AWGN

        [r,l] = xcorr(x,ref);       %% Cross Correlation Detection of Trng Seq  

        y = abs(r(l>-1));       %% Trng @ y(10177)
        %% Apply CFAR Detection procedure:  use Slotted Window noise mean estimator
        norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
        y = y(121:end-50);          %% remove delay and align data and noise level estimate
        norm = norm(121:end-50);
% plot((1:length(y))/OSR,y-Kt*norm);pause(0.5); 
% hold on;
        [pk,kp] = max(y-Kt*norm);       %% Select the peak threshold exceedence 
        if(pk>KD)                       %% Use only if y(kp) > KD*norm(kp)
            if(abs(kp-10056)<3)
                det(k) = det(k) + 1;
            end
        end
        %% FAR detection
        [w,l] = xcorr(noise,ref);       %% Cross Correlation Detection of Trng Seq (Noise only)
        y = abs(w(l>-1));       
        norm = filter([ones(1,32) 0 0 0 0 0 ones(1,32)]/64,1,y);
        y = y(121:end-50);
        norm = norm(121:end-50);
% plot((1:length(y))/OSR,y-Kt*norm,'g--');pause(0.5);
% hold off;
        [pk,kp] = max(y-Kt*norm);
        if(pk>KD)
            far(k) = far(k) + 1;
        end
        
    end
    k = k+1;
end
% plot(SNR,det/Ntrials,'o-');grid on;
% xlabel('Es/No (dB)')
% ylabel('P_D')

loglog(far/Ntrials,det/Ntrials)
ylabel('P_D')
xlabel('P_F_A')
    
figure
plot(far(end:-1:1)/Ntrials,det(end:-1:1)/Ntrials,'r')
ylabel('P_D')
xlabel('P_F_A')
    
 save ROC_BestModZadof_Chu_CurveTrngDet_p0dBorig det far Ntrials;
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%