function [shapingf, training_type, seq_type, data, ref, refshort, reflen, txSig, txSiglen ] = ...
    select_sequenceR3( M, freqsep, nsamp, fs, training_type)
%
% training sequences - generation of sequences for further analysis
% Arunas Macikunas
% updated 9 June 2017
% updates Aug 2017 for Krzysztof and Space Norway sequence 'detectors',
% and AIS 1371 stanard sequence detector AGM
%
if(isempty(training_type))
    training_type = menu('Select Reference','2092 Barker sequence (from Jan)', 'Barker (Jan coder)', ...
        'Dual Barker time reversed AGM', ...
        '13-bit Correlator Barker (Jan coder)','Space Norway 27-bit propsed', 'Zadoff Chu 27-symb', ...
        'Zadoff Chu 32-symb', 'Chu 27-symb', 'Chu 32-symb','Polyphase Barker 27 (TD)','Frank 25', ...
        'Unattributed binary 27 (TD)', 'Unfiltered Zadoff-Chu 27', 'AM 69 phase sequence', ...
        'Barker  (TimD rotated QPSK)', '1371 AIS GMSK sequence (TS+SF)', ...
        '2092 Barker sequence (KB-detector)', 'Dual Barker time reversed AGM (KB-detector)', ...
        'Space Norway 27-bit (differential detector)', 'Space Norway 40-bit (differential detector)',  ...
        '4 FSK like DMR+AGM', '4 FSK Arunas', '4 FSK - Space Norway sequence', '4 FSK Arunas (FFT detector)', ...
        'Future - various to try');
end

pi2                 = 2*pi;
refshort            = [];
txSig               = [];
shapingf            = [];
ref                 = [];
data                = [];

if(training_type==1)
    
    % Copy and paste of the VDES seq generated by my code - pi/4-QPSK:
    ref = [0.70711 + 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, 0.70711 + 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, -1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, -1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, 0.70711 + 0.70711i, ...
        -1.00000 + 0.00000i, -0.70711 - 0.70711i, 1.00000 + 0.00000i, ...
        -0.70711 - 0.70711i, 1.00000 + 0.00000i, -0.70711 - 0.70711i];
    seq_type = 'Barker Manual Pi/4 QPSK 2092 Jan';
    
elseif(training_type==2)
    
    barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG        = [1 barker ~barker];         %% As currently proposed
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    seq_type    = 'Barker pi/4 QPSK Jan';
    
elseif(training_type==3)
    barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    revbarker   = barker(13:-1:1);
    TRNG        = [1 barker revbarker];         %% As currently proposed
    %     TRNG = [barker 3 revbarker];         %% As currently proposed
    TRNG        = [1 ~barker ~revbarker];         %% As currently proposed
    TRNG        = [1 ~barker ~revbarker];         %% As currently proposed -14 dB peak, -3.19 dB integrated
    TRNG        = [1 ~barker ~revbarker 0];      %% As currently proposed -13.2, -3.5 dB 28 symbols
    % above are experiments - not as effective as last alternative
    TRNG        = [ ~barker 1 revbarker ];      %% As currently proposed -14.1, -3.4 dB 28 symbols
    
    % two more cases to try - stretched barker - repeat each symbol twice
    % interleave forward and reversed Barker sequence, and try with one of
    % them inverted
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trng_mapped;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    seq_type    = 'Barker reversed AGM pi/4 QPSK';
    
elseif(training_type==4)
    
    barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG        = [1 barker ~barker];         %% As currently proposed
    TRNGshort   = [barker];
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trngshort_mapped(1:2:(2*length(TRNGshort)-1))   = TRNGshort;
    trngshort_mapped(2:2:2*length(TRNGshort))       = TRNGshort;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    %     [refshort]  = qpskMapperJan(trngshort_mapped);
    [refshort, ~]    = pi4qpskMapper(trngshort_mapped,0);
    
    seq_type    = '13-bit only Barker pi/4 QPSK';
    
elseif(training_type==5)
    
    % New Space Norway Training Sequence
    % from Hans, Mar 9, 2017, HC Haugli 9 March 2017v1.doc
    % comment HCH:  Based on search, looses 0.3% of packets  at Es/N0 of 5 dB
    %  010001010010010000000110011
    %     TRNG = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 1 0 1 1 ];  % 1 bit altered 0 to 1 - AGM to see impact
    
    TRNG        = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    seq_type    = 'Space Norway 27 symb Pi/4 QPSK';
    
elseif(training_type==6)
    
    % zad         = menu('Which ZadofChu 27-bit sequence','Default', 'Unfiltered');
    zad = 2;
    if(zad==1)
        load ZadofChu_Amb;
        ref     = bb;            %
    else
        load ZadofChu_AmbxxUnFilt; ref = bb;    %% "best" Choice so far: Unfiltered
    end
    seq_type    = 'ZadoffChu 27 symb';
    
elseif(training_type==7)
    
    load ZadofChu32_Ambxy;
    ref         = bb;
    seq_type    = 'ZadoffChu32-Ambxy';
    
elseif(training_type==8)
    load CHU_27;
    ref         = a;  % am - does this exist - need to create it
    seq_type    = 'Chu-27';
    
elseif(training_type==9)
    
    load CHU_32;
    ref         = a;  % am - does this exist - need to create it
    seq_type    = 'Chu-32';
    
elseif(training_type==10)
    
    load polyphase_barker27;
    ref         = bb;
    seq_type    = 'Polyphase Barker Pi/4-QPSK';
    
elseif(training_type==11)
    
    Frank25     = [0 0 0 0 0 0 1 2 3 4 0 2 4 1 3 0 3 1 4 2 0 4 3 2 1];
    ref         = exp(1i*pi2*Frank25/5);
    seq_type    = 'Frank25';
    
elseif(training_type==12)
    if(1==1)            %original version - for double length sequence (symbols not duplicated)
        [ref, ~] = pi4qpskMapper([0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1], 0);
    else        % updated mapping, AGM June 2017
        trng_mapped = [0 0 0 1 1 0 1 1 0 0 0 0 1 1 1 1 0 0 0 1 0 0 1 0 0 1 1 0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1 1 1 1 0 0 0 0 1 0 1 1];
        [ref]   = qpskMapperJan(trng_mapped);       % this version used for intersessional in June presentation 13 June 2017
    end
    seq_type    = 'Unatributed binary (TD) Pi/4 QPSK';
    
elseif(training_type==13)
    
    load ZadofChu_AmbxxUnFilt;
    ref         = bb;    %% "best" Choice so far: Unfiltered
    
    seq_type    = 'ZadoffChu-27 Unfiltered';
    
elseif(training_type==14)
    
    load AMphase
    ref         = exp(1i*pi2*phasefn);
    seq_type    = 'AM 69 phase sequence';
    
elseif(training_type==15)
    % JanS: Tim's original code - pi/4 rotated BPSK, this is not pi/4-QPSK!!!!!
    barker = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG = [1 barker ~barker]          %% As currently proposed
    symbols = [TRNG; TRNG];             %% Set-up for phase map
    TrngSym = [2 1]*symbols;
    TrngSym(TrngSym==3) = 2;
    ref = exp(1i*(TrngSym*pi/2+pi/4));          %% pi/4 rotated BPSK
    %     iref = reshape([ref.' zeros(length(ref),OSR-1)].',1,OSR*length(ref));
    %     sref = filter(hsrrc,1,[iref zeros(1,ndelay)]);
    %     sref = sref(ndelay+1:end);
    seq_type = 'Barker pi/4 rotated QPSK';
    
elseif(training_type==16)
    
    AISver = 1;
    %
    % Create 1371 AIS training sequences
    % Most common is 0 1 0 1 training sequence (95%), equal prob for 2 NRZI
    % start states (C. Short, July 2017)
    %
    pretrain = [ 0 0 0 0 0 0 0 0 ];
    training0 = [ 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1];
    training1 = [ 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0];
    startflg = [ 0 1 1 1 1 1 1 0 ];
    
    modulation_idx      = 0.5;
    ramp_up_tau         = 0.4e-3;   %  0.4e-3 provides 90% match to Tau, Chris S. default 0.5e-3
    ramp_up_tau         = 0.2e-3;   %  0.2e-3 provides better visible match to drawing in 1371 standard
    rand_data_len       = 2^5;
    rand_data_len       = 0;
    
    ramp_down_tau       = .2e-4;
    num_non_buffer_bits = 48;
    bitrate             = 9600;
    samplesPerBit       = 4;
    samplesPerBit       = 1;
    
    if(AISver == 1)
        % version 1 (training seq starting with 0, then NRXI startbit false state)
        
        packet=[ pretrain training0 startflg floor(2*rand(1,rand_data_len))];
        NRZI_startbit       = false;   
        
        [signal , t]        = modulate_packetAGM(packet,NRZI_startbit,modulation_idx,ramp_up_tau, ...
            ramp_down_tau, num_non_buffer_bits, bitrate, samplesPerBit);
        
    elseif(AISver == 2)
        
        % version 2 (training seq starting with 0, then NRXI startbit true state)
        
        packet =[ pretrain training0 startflg floor(2*rand(1,rand_data_len))];
        NRZI_startbit       = true;
        
        [signal , t]        = modulate_packetAGM(packet,NRZI_startbit,modulation_idx,ramp_up_tau, ...
            ramp_down_tau, num_non_buffer_bits, bitrate, samplesPerBit);
        
    elseif(AISver == 3)
        
        % version 3 (training seq starting with 1, then NRXI startbit false state)
        
        packet =[ pretrain training1 startflg floor(2*rand(1,rand_data_len))];
        NRZI_startbit       = false;
        
        [signal , t]        = modulate_packetAGM(packet,NRZI_startbit,modulation_idx,ramp_up_tau, ...
            ramp_down_tau, num_non_buffer_bits, bitrate, samplesPerBit);
        
        
    elseif(AISver == 4)
        
        % version 4
        
        packet =[ pretrain training1 startflg floor(2*rand(1,rand_data_len))];
        NRZI_startbit       = true;
        
        [signal , t]        = modulate_packetAGM(packet,NRZI_startbit,modulation_idx,ramp_up_tau, ...
            ramp_down_tau, num_non_buffer_bits, bitrate,samplesPerBit);
    end
    %     [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    ref         = signal(9:end).';
    
    seq_type    = 'AIS 1371 GMSK';
    
elseif(training_type==17)
    
    barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    TRNG        = [1 barker ~barker];         %% As currently proposed
    TRNGshort   = [barker];
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trngshort_mapped(1:2:(2*length(TRNGshort)-1))   = TRNGshort;
    trngshort_mapped(2:2:2*length(TRNGshort))       = TRNGshort;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    %     [refshort]  = qpskMapperJan(trngshort_mapped);
    [refshort, ~]    = pi4qpskMapper(trngshort_mapped,0);
    
    seq_type    = 'Barker pi/4 QPSK KB-detector';
    
elseif(training_type==18)
    barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    revbarker   = barker(13:-1:1);
    %     TRNG        = [1 barker revbarker];         %% As currently proposed
    %     TRNG        = [barker 3 revbarker];         %% As currently proposed
    %     TRNG        = [1 ~barker ~revbarker];         %% As currently proposed
    %     TRNG        = [1 ~barker ~revbarker];         %% As currently proposed -14 dB peak, -3.19 dB integrated
    %     TRNG        = [1 ~barker ~revbarker 0];      %% As currently proposed -13.2, -3.5 dB 28 symbols
    % above are experiments - not as effective as last alternative
    TRNG        = [ ~barker 1 revbarker ];      %% As currently proposed -14.1, -3.4 dB 28 symbols
    
    % two more cases to try - stretched barker - repeat each symbol twice
    % interleave forward and reversed Barker sequence, and try with one of
    % them inverted
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    trng_mapped;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    seq_type    = 'Barker reversed AGM KB-detector';
    
elseif(training_type==19)
    
    TRNG        = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    seq_type    = 'Space Norway 27 symb (differential detector)';
    
elseif(training_type==20)
    
    if(1==0)
        TRNG        = [ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ];
    else
        TRNG = [1];      % new 40 symbol sequence required!  AGM July 2017
    end
    trng_mapped(1:2:(2*length(TRNG)-1)) = TRNG;
    trng_mapped(2:2:2*length(TRNG))     = TRNG;
    
    %     [ref]       = qpskMapperJan(trng_mapped);
    [ref, ~]    = pi4qpskMapper(trng_mapped,0);
    
    seq_type    = 'Space Norway 40 symb (differential detector)';
    
elseif(training_type==21)
    
    data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 ]; % Arunas 4 1st 24-symbols from DMR voice BS bound
    data = [ 1 3 1 1 1 1 3 3 3 1 1 3 3 1 3 3 1 3 1 1 3 3 1 3 0 2 0 ]; % Arunas 5 1st 24-symbols from DMR voice BS bound + AM
    data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
    data = [ 1 3 0 1 2 1 0 3 2 0 1 2 3 1 0 3 1 3 2 1 0 3 1 3 0 2 0 ]; % Arunas 7 1st 24-symbols from DMR voice BS bound + AM
    data = [[ 1 3 1 1 1 1 3 3 3 1 1 3] ([ 3 1 3 3 1 3 1 1 3 3 1 3 ] - 1) [0 1 2] ]; % Arunas 4 1st 24-symbols from DMR voice BS bound
    data = [ 1 3 1 1 0 1 1 3 2 3 3 1 1 3 3 1 3 0 3 1 3 1 1 3 3 1 3 ]; % Arunas 6 1st 24-symbols from DMR voice BS bound + AM -12.9 -3.2
    
    txSig       = fskmod(data, M,freqsep, nsamp, fs);
    %     load txSigref txSig M
    shapingf    = 1;
    
    seq_type    = '4FSK DMR+AGM';
    
elseif(training_type==22)
    
    numbersymb  = 27;
    %     data        = randi([0 M-1],numbersymb,1)';
    data = 2*[ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ]; %space Norway
    data = [ 1 1 1 0 0 0 0 1 1 0 1 1 0 1 1 0 0 1 0 0 1 1 1 1 0 1 0 ]; % Arunas 1
    data = [ 2 1 1 3 0 0 3 1 1 3 1 1 3 1 1 3 0 1 3 0 1 2 1 1 3 1 0 ]; % Arunas BAD - 2          peak side -4.22 (dB), ISLR 2.84 (dB)
    %     data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 1 4 ] - 1; % Arunas 3 good,    peak side -13.97 (dB), ISLR -4.23 (dB)
    %     data = [ 4 3 2 1 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 1 4 ] - 1; % Arunas 3 not bad, peak side -9.39 (dB), ISLR -1.09 (dB)
    %     data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 4 2 3 ] - 1; % Arunas 4 not bad, peak side -10.15 (dB), ISLR -2.53 (dB)
    %     data        = randi([0 M-1],numbersymb,1)';
    data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 3 1 2 ] - 1; % Arunas 5 good, peak side -14.11 (dB), ISLR -4.19 (dB)
    data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 2 1 4 3 ] - 1; % Arunas 6 not bad, peak side -11.22 (dB), ISLR -2.64 (dB)
    data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 3 1 ] - 1; % Arunas 7 great, peak side -16.06 (dB), ISLR -6.12 (dB)
    
    txSig       = fskmod(data, M,freqsep, nsamp, fs);
    shapingf    = 1;
    
    seq_type    = '4FSK Arunas';
    
elseif(training_type==23)
    
    data = 2*[ 0 1 0 0 0 1 0 1 0 0 1 0 0 1 0 0 0 0 0 0 0 1 1 0 0 1 1 ]; % Space Norway applied to FSK
    %     data = 2*[ 1 1 1 1 1 1 1 1 1 1 0 0 0 0 1 1 1 1 0 0 1 1 0 0 1 1 1 ]; % slow BARKER peak side -7.56 (dB), ISLR 1.94 (dB)
    %     barker      = [1 1 1 1 1 0 0 1 1 0 1 0 1];
    
    txSig       = fskmod(data, M,freqsep, nsamp, fs);
    shapingf    = 1;
    seq_type    = '4FSK Space Norway sequence';
    
elseif(training_type==24)
    
    numbersymb  = 27;
    data = [ 1 2 3 4 2 3 4 1 3 4 1 2 4 1 2 3 4 3 2 1 3 2 1 4 2 3 1 ] - 1; % Arunas 7 great, peak side -16.06 (dB), ISLR -6.12 (dB)
    
    txSig       = fskmod(data, M,freqsep, nsamp, fs);
    shapingf    = 1;
    
    seq_type    = '4FSK Arunas (FFT detector)';
    
elseif(training_type==25)
    % ******* Files I don't have from Dr. Tim D. *********
    %     load Max27P2SL_1; ref = pn0;
    %     load Max32P2SL_TAG3; ref = pn0;
    %     load BestZadoff_ChuRev3; ref = bb;
    %       load BestZadoff_ChuRev3; ref = bb;     %% bb Second-BEST - small search range;
    %     load CHU_33;ref = a;                %% CAZAC Sequence
    %     load tempCHUxy;ref = bb;
    %      load tempChu1; ref = bb;              %% Chu BEST;
    % load tempCHUxx1; ref = bb;            %% very Good Intermediate: Unfiltered
    
end

reflen = length(ref);
txSiglen = length(txSig);
end